"""
Common generated classes shared across all resources.

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

import json
import logging
import mimetypes
import os
import random
import re
import time
import typing

from functools import partial
from urllib.parse import quote

import snowflake.core._http_requests

from snowflake.core._generated import rest
from snowflake.core._generated.configuration import Configuration
from snowflake.core._generated.paging import PagedIter
from snowflake.core._thread_pool import get_thread_pool
from snowflake.core.exceptions import APIError, InvalidResponseError, RetryTimeoutError, _APIValueError
from snowflake.core.version import __version__ as VERSION


if typing.TYPE_CHECKING:
    from snowflake.core._root import Root


logger = logging.getLogger(__name__)


class ApiClient:
    """Generic API client for OpenAPI client library builds.

    OpenAPI generic API client. This client handles the client-
    server communication, and is invariant across implementations. Specifics of
    the methods and models for each application are generated from the OpenAPI
    templates.

    NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech
    Do not edit the class manually.

    :param configuration: .Configuration object for this client
    :param header_name: a header to pass when making calls to the API.
    :param header_value: a header value to pass when making calls to
        the API.
    :param cookie: a cookie to include in the header when making calls
        to the API
    """

    def __init__(self, configuration=None, header_name=None, header_value=None, cookie=None):
        # use default configuration if none is provided
        if configuration is None:
            configuration = Configuration.get_default()
        self.configuration = configuration

        self.rest_client = rest.RESTClientObject(configuration)
        self.default_headers = {}
        if header_name is not None:
            self.default_headers[header_name] = header_value
        self.cookie = cookie
        # Set default User-Agent.
        self.user_agent = "python_api/" + VERSION + ""
        self.client_side_validation = configuration.client_side_validation

    @property
    def user_agent(self):
        """User agent for this API client"""
        return self.default_headers["User-Agent"]

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers["User-Agent"] = value

    def set_default_header(self, header_name, header_value):
        self.default_headers[header_name] = header_value

    @staticmethod
    def get_host(root: "Root") -> str:
        if (
            hasattr(root, "connection")
            and root.connection is not None
            and hasattr(root.connection, "rest")
            and root.connection.rest is not None
            and hasattr(root.connection.rest, "_protocol")
            and root._hostname is not None
            and hasattr(root.connection.rest, "_port")
        ):
            return f"{root.connection.rest._protocol}://{root._hostname}:{root.connection.rest._port}"
        # for spark connect's RESTRoot
        if hasattr(root, "_restConnection") and root._restConnection is not None:
            return root._restConnection.url
        return "https://org-account.snowflakecomputing.com"  # default stub value

    _default = None

    @classmethod
    def get_default(cls):
        """Return new instance of ApiClient.

        This method returns newly created, based on default constructor,
        object of ApiClient class or returns a copy of default
        ApiClient.

        :return: The ApiClient object.
        """
        if cls._default is None:
            cls._default = ApiClient()
        return cls._default

    @classmethod
    def set_default(cls, default):
        """Set default instance of ApiClient.

        It stores default ApiClient.

        :param default: object of ApiClient.
        """
        cls._default = default

    def __call_api(
        self,
        root,
        resource_path,
        method,
        path_params=None,
        query_params=None,
        header_params=None,
        body=None,
        post_params=None,
        files=None,
        response_types_map=None,
        auth_settings=None,
        _return_http_data_only=None,
        collection_formats=None,
        _preload_content=True,
        _request_timeout=None,
        _host=None,
        _request_auth=None,
        _deserialize_response_fn=None,
    ):
        # header parameters
        header_params = header_params or {}
        user_agents = ["python_api/" + VERSION + ""]
        if root.root_config.has_user_agents():
            user_agents.append(root.root_config.get_user_agents())
        self.user_agent = " ".join(user_agents)
        header_params.update(self.default_headers)
        if self.cookie:
            header_params["Cookie"] = self.cookie
        if header_params:
            header_params = snowflake.core._http_requests.sanitize_for_serialization(header_params)
            header_params = dict(snowflake.core._http_requests.parameters_to_tuples(header_params, collection_formats))

        # post parameters
        if post_params or files:
            post_params = post_params if post_params else []
            post_params = snowflake.core._http_requests.sanitize_for_serialization(post_params)
            post_params = snowflake.core._http_requests.parameters_to_tuples(post_params, collection_formats)
            post_params.extend(self.files_parameters(files))

        # auth setting
        self.update_params_for_auth(
            header_params, query_params, auth_settings, resource_path, method, body, request_auth=_request_auth
        )

        # body
        if body:
            body = snowflake.core._http_requests.sanitize_for_serialization(body)

        # request url
        if _host is None:
            url = self.get_host(root) + resource_path
        else:
            # use server/host defined in path or operation instead
            url = _host + resource_path

        # query parameters
        if query_params:
            query_params = snowflake.core._http_requests.sanitize_for_serialization(query_params)
            url_query = self.parameters_to_url_query(query_params, collection_formats)
            url += "?" + url_query

        try:
            # perform request and return response, maybe with retry
            response_data = self.request_with_retry(
                root,
                method,
                url,
                query_params=query_params,
                headers=header_params,
                post_params=post_params,
                body=body,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
            )
        except APIError as e:
            if e.body:
                e.body = e.body.decode("utf-8")
            raise e

        self.last_response = response_data

        return_data = response_data

        if not _preload_content:
            return return_data

        response_type = response_types_map.get(str(response_data.status), None)

        # if response_type not in ["file", "bytes"]:
        #     match = None
        #     content_type = response_data.getheader('content-type')
        #     if content_type is not None:
        #         match = re.search(r"charset=([a-zA-Z\-\d]+)[\s;]?", content_type)
        #     encoding = match.group(1) if match else "utf-8"
        #     response_data.data = response_data.data.decode(encoding)

        # deserialize response data

        if response_type:
            large_results_data = ApiClient.large_results(response_data)
            if large_results_data is None:
                # regular, non-large results use case
                return_data = (
                    _deserialize_response_fn(response_data, response_type)
                    if _deserialize_response_fn
                    else response_data
                )
            else:
                # Closure for fetching the next chunk in the PagedIter iterator
                def _fetch_next_chunk(chunk_index, deserialize_type):
                    results_path = large_results_data[0]

                    # The first page may already be in the response body, if it is not just a
                    # message informing us of the large results.
                    if chunk_index == 0 and not ApiClient.has_large_results_message_in_body(response_data.data):
                        chunk_response_data = response_data
                    else:
                        # For now, do this because query_params is not actually being used properly in self.request
                        chunk_url = f"{self.get_host(root)}{results_path}?page={chunk_index}"

                        chunk_response_data = self.request_with_retry(
                            root,
                            "GET",
                            chunk_url,
                            headers=header_params,
                            _preload_content=True,
                            _request_timeout=_request_timeout,
                            query_params={"page": chunk_index},
                        )

                    return (
                        _deserialize_response_fn(chunk_response_data, deserialize_type)
                        if _deserialize_response_fn
                        else chunk_response_data
                    )

                if "Iterable" in response_type:
                    return PagedIter(
                        partial(_fetch_next_chunk, deserialize_type=response_type),
                        number_of_chunks_=large_results_data[1],
                    )
                else:
                    # At most, we should only need to fetch one chunk if it's a point lookup,
                    # i.e., one row return
                    return_data = _fetch_next_chunk(0, response_type)
        else:
            return_data = None

        if _return_http_data_only:
            return return_data
        else:
            return (return_data, response_data.status, response_data.getheaders())

    def call_api(
        self,
        root,
        resource_path,
        method,
        path_params=None,
        query_params=None,
        header_params=None,
        body=None,
        post_params=None,
        files=None,
        response_types_map=None,
        auth_settings=None,
        async_req=None,
        _return_http_data_only=None,
        collection_formats=None,
        _preload_content=True,
        _request_timeout=None,
        _host=None,
        _request_auth=None,
        _deserialize_response_fn=None,
    ):
        """Makes the HTTP request (synchronous) and returns deserialized data.

        To make an async_req request, set the async_req parameter.

        :param resource_path: Path to method endpoint.
        :param method: Method to call.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be
            placed in the request header.
        :param body: Request body.
        :param post_params dict: Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param auth_settings list: Auth Settings names for the request.
        :param response: Response data type.
        :param files dict: key -> filename, value -> filepath,
            for `multipart/form-data`.
        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
                                       and headers
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
                                 be returned without reading/decoding response
                                 data. Default is True.
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :param _deserialize_response_fn: function to deserialize the response data
        :type _request_token: dict, optional
        :return:
            If async_req parameter is True,
            the request will be called asynchronously.
            The method will return the request thread.
            If parameter async_req is False or missing,
            then the method will return the response directly.
        """
        # Insert path parameters to fully resolve URL before logging
        resource_path = snowflake.core._http_requests.resolve_url(
            resource_path,
            path_params,
            collection_formats,
            self.configuration.safe_chars_for_path_param,
        )
        logger.info(
            "performing %s HTTP %s call to %s",
            "an async" if async_req else "a",
            method,
            resource_path,
        )
        logger.debug(
            "request has path params: %r, query_params: %r, header params: %r and body: %r",
            path_params,
            query_params,
            header_params,
            body,
        )
        if not async_req:
            return self.__call_api(
                root,
                resource_path,
                method,
                path_params,
                query_params,
                header_params,
                body,
                post_params,
                files,
                response_types_map,
                auth_settings,
                _return_http_data_only,
                collection_formats,
                _preload_content,
                _request_timeout,
                _host,
                _request_auth,
                _deserialize_response_fn,
            )

        pool = get_thread_pool(root.parameters().max_threads)
        return pool.submit(
            self.__call_api,
            *(
                root,
                resource_path,
                method,
                path_params,
                query_params,
                header_params,
                body,
                post_params,
                files,
                response_types_map,
                auth_settings,
                _return_http_data_only,
                collection_formats,
                _preload_content,
                _request_timeout,
                _host,
                _request_auth,
                _deserialize_response_fn,
            ),
        )

    def request_with_retry(
        self,
        root,
        method,
        url,
        query_params=None,
        headers=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        """Response time by default one hour"""
        enter_timing = time.time()
        response_data = self.request(
            root,
            method,
            url,
            query_params=query_params,
            headers=headers,
            post_params=post_params,
            body=body,
            _preload_content=_preload_content,
            _request_timeout=_request_timeout,
        )

        if not root.parameters().should_retry_request or response_data.status not in rest.RETRY_STATUS_CODES:
            return response_data

        logger.info(
            f"Request Status {response_data.status} ({snowflake.core._http_requests.STATUS_CODES_MAPPING.get(response_data.status)}) Retrying..."
        )
        # Query retry support
        result_endpoint = response_data.getheader("Location")
        if response_data.status == 202 and result_endpoint is None:
            raise InvalidResponseError(
                f"{snowflake.core._http_requests.STATUS_CODES_MAPPING.get(response_data.status)} result endpoint is missing"
            )

        if _request_timeout is None:
            _request_timeout = snowflake.core._http_requests.DEFAULT_RETRY_TIMEOUT_SECONDS
        wait_for_results_timeout = enter_timing + _request_timeout

        exponential_wait_time = 1  # wait time increases exponentially, 30% more everytime
        while True:
            time_remaining = wait_for_results_timeout - time.time()
            if time_remaining <= 0:
                break
            # Full Jitter
            # ref: https://aws.amazon.com/blogs/architecture/exponential-backoff-and-jitter/
            # Take max(1, wait_time) to avoid making too many requests in a short period
            wait_time = max(1, random.randint(0, int(min(exponential_wait_time, time_remaining))))

            if response_data.status in rest.RETRY_STATUS_CODES:
                logger.info(
                    f"{snowflake.core._http_requests.STATUS_CODES_MAPPING.get(response_data.status)} detected. Sleeping for {wait_time} seconds before checking again"
                )

                time.sleep(wait_time)

                new_url = url
                if response_data.status == 202:
                    new_url = self.get_host(root) + result_endpoint
                    method = "GET"

                response_data = self.request(
                    root,
                    method,
                    new_url,
                    query_params=query_params,
                    headers=headers,
                    post_params=post_params,
                    body=body,
                    _preload_content=_preload_content,
                    _request_timeout=max(time_remaining - wait_time, 1),
                    # request_timeout can never be zero
                )
            else:
                logger.info(
                    f"{snowflake.core._http_requests.STATUS_CODES_MAPPING.get(response_data.status)} query completed"
                )
                return response_data

            exponential_wait_time *= 1.3

        raise RetryTimeoutError(
            f"{response_data.status} {snowflake.core._http_requests.STATUS_CODES_MAPPING.get(response_data.status)} query timeout"
        )

    def request(
        self,
        root,
        method,
        url,
        query_params=None,
        headers=None,
        post_params=None,
        body=None,
        _preload_content=True,
        _request_timeout=None,
    ):
        """Makes the HTTP request using RESTClient."""
        if method == "GET":
            return self.rest_client.get_request(
                root,
                url,
                query_params=query_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                headers=headers,
            )
        elif method == "HEAD":
            return self.rest_client.head_request(
                root,
                url,
                query_params=query_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                headers=headers,
            )
        elif method == "OPTIONS":
            return self.rest_client.options_request(
                root,
                url,
                query_params=query_params,
                headers=headers,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
            )
        elif method == "POST":
            return self.rest_client.post_request(
                root,
                url,
                query_params=query_params,
                headers=headers,
                post_params=post_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        elif method == "PUT":
            try:
                return self.rest_client.put_request(
                    root,
                    url,
                    query_params=query_params,
                    headers=headers,
                    post_params=post_params,
                    _preload_content=_preload_content,
                    _request_timeout=_request_timeout,
                    body=body,
                )
            except APIError as error:
                # Raise a more helpful user error if CoA is not supported for this resource;
                # this is represented as either 405 or 501 on the server.
                if error.status in (405, 501):
                    raise NotImplementedError("create_or_update is not yet supported for this resource.")
                raise

        elif method == "PATCH":
            return self.rest_client.patch_request(
                root,
                url,
                query_params=query_params,
                headers=headers,
                post_params=post_params,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        elif method == "DELETE":
            return self.rest_client.delete_request(
                root,
                url,
                query_params=query_params,
                headers=headers,
                _preload_content=_preload_content,
                _request_timeout=_request_timeout,
                body=body,
            )
        else:
            raise _APIValueError("http method must be `GET`, `HEAD`, `OPTIONS`, `POST`, `PATCH`, `PUT` or `DELETE`.")

    def parameters_to_url_query(self, params, collection_formats):
        """Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: URL query string (e.g. a=Hello%20World&b=123)
        """
        new_params = []
        if collection_formats is None:
            collection_formats = {}
        for k, v in params.items() if isinstance(params, dict) else params:
            if isinstance(v, (int, float)):
                v = str(v)
            if isinstance(v, bool):
                v = str(v).lower()

            if k in collection_formats:
                collection_format = collection_formats[k]
                if collection_format == "multi":
                    new_params.extend((k, value) for value in v)
                else:
                    if collection_format == "ssv":
                        delimiter = " "
                    elif collection_format == "tsv":
                        delimiter = "\t"
                    elif collection_format == "pipes":
                        delimiter = "|"
                    else:  # csv is the default
                        delimiter = ","
                    new_params.append((k, delimiter.join(quote(str(value)) for value in v)))
            else:
                new_params.append((k, v))

        return "&".join(["=".join(map(str, item)) for item in new_params])

    def files_parameters(self, files=None):
        """Builds form parameters.

        :param files: File parameters.
        :return: Form parameters with files.
        """
        params = []

        if files:
            for k, v in files.items():
                if not v:
                    continue
                file_names = v if type(v) is list else [v]
                for n in file_names:
                    with open(n, "rb") as f:
                        filename = os.path.basename(f.name)
                        filedata = f.read()
                        mimetype = mimetypes.guess_type(filename)[0] or "application/octet-stream"
                        params.append(tuple([k, tuple([filename, filedata, mimetype])]))

        return params

    def select_header_accept(self, accepts):
        """Returns `Accept` based on an array of accepts provided.

        :param accepts: List of headers.
        :return: Accept (e.g. application/json).
        """
        if not accepts:
            return

        for accept in accepts:
            if re.search("json", accept, re.IGNORECASE):
                return accept

        return accepts[0]

    def select_header_content_type(self, content_types):
        """Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types.
        :return: Content-Type (e.g. application/json).
        """
        if not content_types:
            return None

        for content_type in content_types:
            if re.search("json", content_type, re.IGNORECASE):
                return content_type

        return content_types[0]

    def update_params_for_auth(self, headers, queries, auth_settings, resource_path, method, body, request_auth=None):
        """Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param queries: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        :resource_path: A string representation of the HTTP request resource path.
        :method: A string representation of the HTTP request method.
        :body: A object representing the body of the HTTP request.
        The object type is the return value of sanitize_for_serialization().
        :param request_auth: if set, the provided settings will
                             override the token in the configuration.
        """
        if not auth_settings:
            return

        if request_auth:
            self._apply_auth_params(headers, queries, resource_path, method, body, request_auth)
            return

        for auth in auth_settings:
            auth_setting = self.configuration.auth_settings().get(auth)
            if auth_setting:
                self._apply_auth_params(headers, queries, resource_path, method, body, auth_setting)

    def _apply_auth_params(self, headers, queries, resource_path, method, body, auth_setting):
        """Updates the request parameters based on a single auth_setting

        :param headers: Header parameters dict to be updated.
        :param queries: Query parameters tuple list to be updated.
        :resource_path: A string representation of the HTTP request resource path.
        :method: A string representation of the HTTP request method.
        :body: A object representing the body of the HTTP request.
        The object type is the return value of sanitize_for_serialization().
        :param auth_setting: auth settings for the endpoint
        """
        if auth_setting["in"] == "cookie":
            headers["Cookie"] = auth_setting["value"]
        elif auth_setting["in"] == "header":
            if auth_setting["type"] != "http-signature":
                headers[auth_setting["key"]] = auth_setting["value"]

        elif auth_setting["in"] == "query":
            queries.append((auth_setting["key"], auth_setting["value"]))
        else:
            raise _APIValueError("Authentication token must be in `query` or `header`")

    @staticmethod
    def has_large_results_message_in_body(body):
        try:
            result = json.loads(body)
            return (
                "result_handler" in result
                and "message" in result
                and "Large result set. Use provided Link" in result["message"]
            )
        except ValueError:
            return False

    @staticmethod
    def large_results(response):
        """Returns a tuple of the result URL and number of chunks; returns None if no large results are detected.

        :param response: A RESTResponse object in which to check for large results.
        :return: A Tuple of (results_url, number_of_chunks) if there are large results, or else None.
        """
        # This should be the normal way in which we figure out where to get the results from,
        # as well as how many chunks there are to get: check for "Links" in the headers.
        if "Link" in response.getheaders():
            return ApiClient.get_path_and_chunk_count_from_header(response.getheaders()["Link"])

        # Due to a bug, we use the alternate logic to infer the URL from the UUID
        if ApiClient.has_large_results_message_in_body(response.data):
            result = json.loads(response.data)

            handler_id = result["result_handler"]
            results_path = "/api/v2/results/" + handler_id

            # If there is no "Link" header, there is just one chunk
            num_chunks = 1

            return results_path, num_chunks

        return None

    @staticmethod
    def get_path_and_chunk_count_from_header(links_str):
        links_list = links_str.split(",")

        def parse_links(s):
            import re

            # Use regex to extract necessary parts
            #
            # Explanation:
            # The result links are provided to us in the form:
            # '</api/v2/results/01b39664-0100-0001-0000-0000000430391?page=0>; rel="first"'
            #
            # We wish to find how many chunks there are by finding the one that has rel="last",
            # therefore we find page index corresponding to the link with that rel value.
            #
            # 1. <(.*?)> matches and captures the URL inside the angle brackets
            # 2. page=(\d+) matches 'page=' followed by one or more digits to find the page number
            # 3. rel="([^"]*)" matches 'rel="'
            pattern = r'<(.*?)\?page=(\d+)>; rel="([^"]*)"'

            # Search using the regular expression
            match = re.search(pattern, s)
            if match:
                parse_result = dict()
                parse_result["url"], parse_result["page_number"], parse_result["rel_value"] = match.groups()
                return parse_result

            return None

        parsed_links = [parse_links(link) for link in links_list]

        # Find the last one
        last_link = list(filter(lambda link: link["rel_value"].lower() == "last", parsed_links)).pop()

        # Return the URL; the number of chunks is the chunk index of the last page plus one
        return last_link["url"], int(last_link["page_number"]) + 1


class StoredProcApiClient(ApiClient):
    def __init__(self, root, configuration=None, header_name=None, header_value=None, cookie=None):
        ApiClient.__init__(self, configuration, header_name, header_value, cookie)
        self.rest_client = rest.StoredProcClientObject()



import copy

import pytest

from tests.integ.utils import random_string

from .conftest import test_sequence_template


@pytest.fixture(scope="module")
def sequences_extended(sequences):
    names_list = []

    for _ in range(5):
        names_list.append(random_string(10, "test_sequence_iter_a_"))

    for _ in range(7):
        names_list.append(random_string(10, "test_sequence_iter_b_"))

    for _ in range(3):
        names_list.append(random_string(10, "test_sequence_iter_c_"))

    try:
        for name in names_list:
            sequence = copy.deepcopy(test_sequence_template)
            sequence.name = name
            sequences.create(sequence)

        yield sequences
    finally:
        for name in names_list:
            sequences[name].drop(if_exists=True)


def test_iter_raw(sequences_extended):
    assert len(list(sequences_extended.iter())) >= 15


def test_iter_like(sequences_extended):
    assert len(list(sequences_extended.iter(like="test_sequence_iter_a_%"))) == 5
    assert len(list(sequences_extended.iter(like="test_sequence_iter_b_%"))) == 7
    assert len(list(sequences_extended.iter(like="test_sequence_iter_c_%"))) == 3
    assert len(list(sequences_extended.iter(like="TEST_SEQUENCE_ITER_C_%"))) == 3
    assert len(list(sequences_extended.iter(like="nonexistent_pattern_%"))) == 0



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.dynamic_table._generated.api.dynamic_table_api import DynamicTableApi

__all__ = [
    "DynamicTableApi",
]



from typing import TYPE_CHECKING, Any, Optional

from snowflake.core import PollingOperation
from snowflake.core._common import SchemaObjectCollectionParent, SchemaObjectReferenceMixin
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._operation import PollingOperations
from snowflake.core.cortex.search_service._generated.api import CortexSearchServiceApi
from snowflake.core.cortex.search_service._generated.models import QueryRequest, QueryResponse


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class CortexSearchServiceCollection(SchemaObjectCollectionParent["CortexSearchServiceResource"]):
    """Represents the collection operations of the Snowflake Cortex Search Service resource."""

    def __init__(self, schema: "SchemaResource") -> None:
        super().__init__(schema, CortexSearchServiceResource)
        self._api = CortexSearchServiceApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def search(self, name: str, query: QueryRequest) -> QueryResponse:
        return self._api.query_cortex_search_service(
            self.database.name, self.schema.name, name, query_request=query, async_req=False
        )

    @api_telemetry
    def search_async(self, name: str, query: QueryRequest) -> PollingOperation[QueryResponse]:
        """An asynchronous version of :func:`search`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.query_cortex_search_service(
            self.database.name, self.schema.name, name, query_request=query, async_req=True
        )
        return PollingOperations.identity(future)


class CortexSearchServiceResource(SchemaObjectReferenceMixin[CortexSearchServiceCollection]):
    def __init__(self, name: str, collection: CortexSearchServiceCollection) -> None:
        self.name = name
        self.collection = collection

    @api_telemetry
    def search(
        self,
        query: Optional[str] = None,
        columns: Optional[list[str]] = None,
        filter: Optional[dict[str, Any]] = None,
        limit: int = 10,
        **kwargs: Any,
    ) -> QueryResponse:
        return self.collection._api.query_cortex_search_service(
            self.database.name,
            self.schema.name,
            self.name,
            QueryRequest.from_dict({"query": query, "columns": columns, "filter": filter, "limit": limit, **kwargs}),
            async_req=False,
        )

    @api_telemetry
    def search_async(
        self,
        query: Optional[str] = None,
        columns: Optional[list[str]] = None,
        filter: Optional[dict[str, Any]] = None,
        limit: int = 10,
        **kwargs: Any,
    ) -> PollingOperation[QueryResponse]:
        """An asynchronous version of :func:`search`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.query_cortex_search_service(
            self.database.name,
            self.schema.name,
            self.name,
            QueryRequest.from_dict({"query": query, "columns": columns, "filter": filter, "limit": limit, **kwargs}),
            async_req=True,
        )
        return PollingOperations.identity(future)



# coding: utf-8

# flake8: noqa

"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.stage._generated.api.stage_api import StageApi

# import models into sdk package
from snowflake.core.stage._generated.models.aws_credentials import AwsCredentials
from snowflake.core.stage._generated.models.azure_credentials import AzureCredentials
from snowflake.core.stage._generated.models.credentials import Credentials
from snowflake.core.stage._generated.models.error_response import ErrorResponse
from snowflake.core.stage._generated.models.file_transfer_material import FileTransferMaterial
from snowflake.core.stage._generated.models.presigned_url_request import PresignedUrlRequest
from snowflake.core.stage._generated.models.stage import Stage
from snowflake.core.stage._generated.models.stage_directory_table import StageDirectoryTable
from snowflake.core.stage._generated.models.stage_encryption import StageEncryption
from snowflake.core.stage._generated.models.stage_file import StageFile
from snowflake.core.stage._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.stage._generated.models.success_response import SuccessResponse



from typing import TYPE_CHECKING

from pydantic import StrictStr

from ._generated.api.view_api_base import ViewCollectionBase, ViewResourceBase


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


class ViewCollection(ViewCollectionBase):
    """Represents the collection operations on the Snowflake View resource.

    With this collection, you can create, iterate through, and search for views that you have access to in the
    current context.

    Examples
    ________
    Creating a view instance:

    >>> views = root.databases["my_db"].schemas["my_schema"].views
    >>> new_view = View(
    ...     name="my_view",
    ...     columns=[ViewColumn(name="col1"), ViewColumn(name="col2"), ViewColumn(name="col3")],
    ...     query="SELECT * FROM my_table",
    ... )
    >>> views.create(new_view)
    """

    def __init__(self, schema: "SchemaResource"):
        super().__init__(schema, ViewResource)


class ViewResource(ViewResourceBase):
    """Represents a reference to a Snowflake view.

    With this view reference, you can fetch information about a view, as well as perform certain
    actions on it.
    """

    _plural_name = "views"

    def __init__(self, name: StrictStr, collection: ViewCollection) -> None:
        super().__init__(name, collection)



from collections.abc import Iterator
from typing import TYPE_CHECKING, Optional

from pydantic import StrictStr

from snowflake.core import PollingOperation
from snowflake.core._common import AccountObjectCollectionParent, CreateMode, ObjectReferenceMixin
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._internal.utils import deprecated
from snowflake.core._operation import PollingOperations
from snowflake.core._utils import (
    tag_assignment_to_tag_tuple,
    tag_resource_to_tag_reference,
    tag_tuple_to_tag_assignment,
)
from snowflake.core.tag import TagResource, TagValue
from snowflake.core.warehouse._generated import TagAssignment, TagReference
from snowflake.core.warehouse._generated.api import WarehouseApi
from snowflake.core.warehouse._generated.models.warehouse import WarehouseModel as Warehouse


if TYPE_CHECKING:
    from snowflake.core import Root


class WarehouseCollection(AccountObjectCollectionParent["WarehouseResource"]):
    """Represents the collection operations of the Snowflake Warehouse resource.

    With this collection, you can create, iterate through, and search for warehouses that you have access to
    in the current context.

    Examples
    ________
    Creating a warehouse instance:

    >>> warehouses = root.warehouses
    >>> new_wh = Warehouse(name="my_wh", warehouse_size="XSMALL")
    >>> warehouses.create(new_wh)
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, ref_class=WarehouseResource)
        self._api = WarehouseApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(self, warehouse: Warehouse, *, mode: CreateMode = CreateMode.error_if_exists) -> "WarehouseResource":
        """Create a warehouse in Snowflake.

        Parameters
        __________
        warehouse: Warehouse
        mode: CreateMode, optional
            One of the below enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError` if the
            warehouse already exists in Snowflake. Equivalent to SQL ``create warehouse <name> ...``.

            ``CreateMode.or_replace``: Replace if the warehouse already exists in Snowflake. Equivalent to SQL
            ``create or replace warehouse <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the warehouse already exists in Snowflake. Equivalent to SQL
            ``create warehouse <name> if not exists...``

            Default value is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a warehouse in Snowflake and getting the reference to it:

        >>> warehouse_parameters = Warehouse(
        ...     name="your-warehouse-name",
        ...     warehouse_size="SMALL",
        ...     auto_suspend=500,
        ...)

        >>> # Use the warehouse collection created before to create a reference to a warehouse resource
        >>> # in Snowflake.
        >>> warehouse_reference = warehouse_collection.create(warehouse_parameters)
        """
        real_mode = CreateMode[mode].value
        self._api.create_warehouse(warehouse._to_model(), StrictStr(real_mode), async_req=False)
        return self[warehouse.name]

    @api_telemetry
    def create_async(
        self, warehouse: Warehouse, *, mode: CreateMode = CreateMode.error_if_exists
    ) -> PollingOperation["WarehouseResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        real_mode = CreateMode[mode].value
        future = self._api.create_warehouse(warehouse._to_model(), StrictStr(real_mode), async_req=True)
        return PollingOperation(future, lambda _: self[warehouse.name])

    @api_telemetry
    def iter(self, *, like: Optional[str] = None) -> Iterator[Warehouse]:
        """Iterate through ``Warehouse`` objects in Snowflake, filtering on any optional `like` pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).

        Examples
        ________
        Showing all warehouses that you have access to see:

        >>> warehouses = warehouse_collection.iter()

        Showing information of the exact warehouse you want to see:

        >>> warehouses = warehouse_collection.iter(like="your-warehouse-name")

        Showing warehouses starting with 'your-warehouse-name-':

        >>> warehouses = warehouse_collection.iter(like="your-warehouse-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for warehouse in warehouses:
        >>>     print(warehouse.name, warehouse.warehouse_size)
        """
        warehouses = self._api.list_warehouses(StrictStr(like) if like is not None else None, async_req=False)

        return map(Warehouse._from_model, iter(warehouses))

    @api_telemetry
    def iter_async(self, *, like: Optional[str] = None) -> PollingOperation[Iterator[Warehouse]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_warehouses(StrictStr(like) if like is not None else None, async_req=True)
        return PollingOperation(future, lambda rest_models: map(Warehouse._from_model, iter(rest_models)))


class WarehouseResource(ObjectReferenceMixin[WarehouseCollection]):
    """Represents a reference to a Snowflake warehouse.

    With this warehouse reference, you can fetch information about a warehouse, as well as perform
    certain actions on it.
    """

    def __init__(self, name: str, collection: WarehouseCollection):
        self.name = name
        self.collection = collection

    @api_telemetry
    @deprecated("create_or_alter")
    def create_or_update(self, warehouse: Warehouse) -> None:
        self.create_or_alter(warehouse=warehouse)

    @api_telemetry
    def create_or_alter(self, warehouse: Warehouse) -> None:
        """Create a warehouse in Snowflake or alter one if it already exists.

        Parameters
        __________
        warehouse: Warehouse
            An instance of :class:`Warehouse`.

        Examples
        ________
        Creating or updating a warehouse in Snowflake:

        >>> warehouse_parameters = Warehouse(
        ...     name="your-warehouse-name",
        ...     warehouse_size="SMALL",
        ...     auto_suspend=500,
        ...)

        # Using a ``WarehouseCollection`` to create or update a warehouse in Snowflake:
        >>> root.warehouses["your-warehouse-name"].create_or_alter(warehouse_parameters)
        """
        self.collection._api.create_or_alter_warehouse(warehouse.name, warehouse._to_model(), async_req=False)

    @api_telemetry
    def create_or_alter_async(self, warehouse: Warehouse) -> PollingOperation[None]:
        """An asynchronous version of :func:`create_or_alter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.create_or_alter_warehouse(warehouse.name, warehouse._to_model(), async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def suspend(self, if_exists: Optional[bool] = None) -> None:
        """Suspend the warehouse.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this warehouse before suspending it.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Using a warehouse reference to suspend a warehouse, erroring if it does not exist:

        >>> warehouse_reference.suspend()

        Using a warehouse reference to suspend a warehouse if it exists:

        >>> warehouse_reference.suspend(if_exists=True)
        """
        self.collection._api.suspend_warehouse(self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def suspend_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`suspend`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.suspend_warehouse(self.name, if_exists=if_exists, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def resume(self, if_exists: Optional[bool] = None) -> None:
        """Resume the warehouse.

        Parameters
        __________
        if_exists: bool, optional
            If set to True, the function will not throw an error if the warehouse does not exist.
            The default is ``None``, which is equivalent to False.

        Examples
        ________
        Using a warehouse reference to resume a warehouse, erroring if it does not exist:

        >>> warehouse_reference.resume()

        Using a warehouse reference to resume a warehouse if it exists:

        >>> warehouse_reference.resume(if_exists=True)
        """
        self.collection._api.resume_warehouse(self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def resume_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`resume`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.resume_warehouse(self.name, if_exists=if_exists, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    @deprecated("drop")
    def delete(self, if_exists: Optional[bool] = None) -> None:
        self.drop(if_exists=if_exists)

    @api_telemetry
    def drop(self, if_exists: Optional[bool] = None) -> None:
        """Drop this warehouse.

        Parameters
        __________
        if_exists: bool, optional
            If set to True, the function will not throw an error if the warehouse does not exist.
            The default is ``None``, which is equivalent to False.

        Examples
        ________
        Dropping a warehouse using its reference, erroring if it does not exist:

        >>> warehouse_reference.drop()

        Dropping a warehouse using its reference, if it exists:

        >>> warehouse_reference.drop(if_exists=True)
        """
        self.collection._api.delete_warehouse(self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def drop_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.delete_warehouse(self.name, if_exists=if_exists, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def fetch(self) -> Warehouse:
        """Fetch the details of a warehouse.

        Examples
        ________
        Fetching a warehouse using its reference:

        >>> warehouse = warehouse_reference.fetch()

        # Accessing information of the warehouse with warehouse instance.
        >>> print(warehouse.name, warehouse.warehouse_size)
        """
        return Warehouse._from_model(self.collection._api.fetch_warehouse(self.name, async_req=False))

    @api_telemetry
    def fetch_async(self) -> PollingOperation[Warehouse]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_warehouse(self.name, async_req=True)
        return PollingOperation(future, lambda rest_model: Warehouse._from_model(rest_model))

    @api_telemetry
    def rename(self, new_name: str, if_exists: Optional[bool] = None) -> None:
        """Rename this warehouse.

        This function will ignore other parameters in warehouse instance; use `create_or_update()` to update parameters.

        Parameters
        __________
        new_name: Warehouse
            An instance of :class:`Warehouse`.
        if_exists: bool, optional
            If set to ``True``, the function will not throw an error if the warehouse does not exist.
            The default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Using a warehouse reference to rename a warehouse in Snowflake, erroring if it does not exist:
        >>>  warehouse_reference.rename("new_wh_name")

        Using a warehouse reference to rename a warehouse in Snowflake, if it exists:
        >>>  warehouse_reference.rename("new_wh_name", if_exists=True)
        """
        self.collection._api.rename_warehouse(
            self.name, Warehouse(new_name)._to_model(), if_exists=if_exists, async_req=False
        )

    @api_telemetry
    def rename_async(self, new_name: str, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`rename`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.rename_warehouse(
            self.name, Warehouse(new_name)._to_model(), if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def abort_all_queries(self, if_exists: Optional[bool] = None) -> None:
        """Abort all queries running or queueing on this warehouse.

        Parameters
        __________
        if_exists: bool, optional
            If set to True, the function will not throw an error if the warehouse does not exist.
            The default is ``None``, which is equivalent to False.

        Examples
        ________
        Using a warehouse reference to abort all queries, erroring if it does not exist:

        >>> warehouse = warehouse_reference.abort_all_queries()

        Using a warehouse reference to abort all queries, if it exists:
        >>> warehouse = warehouse_reference.abort_all_queries(if_exists=True)
        """
        self.collection._api.abort_all_queries_on_warehouse(self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def abort_all_queries_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`abort_all_queries`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.abort_all_queries_on_warehouse(self.name, if_exists=if_exists, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def use_warehouse(self) -> None:
        """Use this warehouse as the current warehouse.

        Examples
        ________
        Using a warehouse reference to set the current active warehouse:

        >>> warehouse_reference.use_warehouse()
        """
        self.collection._api.use_warehouse(self.name, async_req=False)

    @api_telemetry
    def use_warehouse_async(self) -> PollingOperation[None]:
        """An asynchronous version of :func:`use_warehouse`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.use_warehouse(self.name, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on a warehouse.

        Parameters
        __________
        tags: dict[TagResource, TagValue]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.set_tags(
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from a warehouse.

        Parameters
        __________
        tag_resources: set[TagResource]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.unset_tags(
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Get the tag assignments for a warehouse.

        Returns all tags assigned to a warehouse. This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Parameter that specifies whether tag assignments inherited by the object from its ancestors in securable
            object hierarchy should be returned as well: - `true`: All tags assigned to this object should be returned,
            inheritance included. - `false`: Only tags explicitly assigned to this object should be returned.
        """
        tag_assignments = self.collection._api.get_tags(
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(name=self.name, with_lineage=with_lineage, async_req=True)
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )


__all__ = ["Warehouse", "WarehouseCollection", "WarehouseResource"]



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.

"""Manages Snowflake Stages.

Example:
    >>> stages: StageCollection = root.databases["mydb"].schemas["myschema"].stages
    >>> mystage = stages.create(Stage("mystage"))
    >>> stage_iter = stages.iter(like="my%")
    >>> mystage = stages["mystage"]
    >>> an_existing_stage = stages["an_existing_stage"]

Refer to :class:`snowflake.core.Root` to create the ``root``.
"""

from ..stage._generated.models import (
    AwsCredentials,
    AzureCredentials,
    Credentials,
    FileTransferMaterial,
    PresignedUrlRequest,
    Stage,
    StageDirectoryTable,
    StageEncryption,
    StageFile,
)
from ._stage import StageCollection, StageResource


__all__ = [
    "Stage",
    "StageResource",
    "StageCollection",
    "AwsCredentials",
    "AzureCredentials",
    "Credentials",
    "FileTransferMaterial",
    "PresignedUrlRequest",
    "StageDirectoryTable",
    "StageEncryption",
    "StageFile",
]



# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.


import pytest

from snowflake.core._common import CreateMode
from snowflake.core._internal.utils import normalize_and_unquote_name
from snowflake.core.warehouse import Warehouse

from ..utils import random_string


pytestmark = pytest.mark.usefixtures("backup_warehouse_fixture")


def test_create(warehouses, session):
    warehouse_name = random_string(5, "test_create_warehouse_")
    test_warehouse = Warehouse(name=warehouse_name, warehouse_size="SMALL", auto_suspend=500)

    warehouse_ref = None
    try:
        # Test warehouse create.
        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.error_if_exists)
        warehouse = warehouse_ref.fetch()
        assert warehouse_name.upper() == warehouse.name.upper()
        assert warehouse.size.upper() == "SMALL"
        assert warehouse.auto_suspend == 500
    finally:
        warehouse_ref.drop()

    test_warehouse = Warehouse(name=warehouse_name, warehouse_size="SMALL", auto_suspend=300)

    try:
        # Test warehouse create.
        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.or_replace)
        warehouse = warehouse_ref.fetch()
        assert warehouse_name.upper() == warehouse.name.upper()
        assert warehouse.size.upper() == "SMALL"
        assert warehouse.auto_suspend == 300
    finally:
        warehouse_ref.drop()

    warehouse_name = random_string(5, "test_create_warehouse_")
    warehouse_name = f'"{warehouse_name}"'
    test_warehouse = Warehouse(name=warehouse_name, warehouse_size="SMALL", auto_suspend=500)

    warehouse_ref = None
    try:
        # Test warehouse create.
        warehouse_ref = warehouses.create(test_warehouse, mode=CreateMode.error_if_exists)
        warehouse = warehouse_ref.fetch()
        assert normalize_and_unquote_name(warehouse_name) == warehouse.name
        assert warehouse.size.upper() == "SMALL"
        assert warehouse.auto_suspend == 500
    finally:
        warehouse_ref.drop()



import pytest

from snowflake.core._internal.utils import normalize_and_unquote_name


pytestmark = [pytest.mark.min_sf_ver("8.35.0")]


def test_list_pipe(pipes, template_pipe, template_pipe_case_sensitive):
    try:
        pipe_resources = []
        pipe_resources += [pipes.create(template_pipe)]
        pipe_names = [pipe.name for pipe in pipes.iter()]
        assert template_pipe.name.upper() in pipe_names

        # check case sensitive pipe name
        pipe_resources += [pipes.create(template_pipe_case_sensitive)]
        pipe_names = [pipe.name for pipe in pipes.iter()]
        assert normalize_and_unquote_name(template_pipe_case_sensitive.name) in pipe_names
        assert template_pipe.name.upper() in pipe_names

        pipe_names = [pipe.name for pipe in pipes.iter(like="test_create_pipe_%")]
        pipe_names = [pipe.name for pipe in pipes.iter()]
        assert normalize_and_unquote_name(template_pipe_case_sensitive.name) in pipe_names
        assert template_pipe.name.upper() in pipe_names

        pipe_names = [pipe.name for pipe in pipes.iter(like="teST_create_pipe_%")]
        assert normalize_and_unquote_name(template_pipe_case_sensitive.name) in pipe_names
        assert template_pipe.name.upper() in pipe_names

        pipe_names = [pipe.name for pipe in pipes.iter(like="%RANDOM%")]
        assert len(pipe_names) == 0
    finally:
        for pipe_resource in pipe_resources:
            pipe_resource.drop(if_exists=True)



from contextlib import suppress

import pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.table import Table, TableColumn


@pytest.fixture()
def populate_tables_for_procedure_call(tables, cursor):
    sql_test_table = "invoices"
    test_table = Table(
        name=sql_test_table, columns=[TableColumn(name="id", datatype="int"), TableColumn(name="price", datatype="int")]
    )
    test_table_handle = tables.create(test_table)
    cursor.execute(f"insert into {sql_test_table} (id, price) values (1, 1), (1, 2), (2, 3), (3, 10)")
    try:
        yield test_table_handle
    finally:
        with suppress(NotFoundError):
            test_table_handle.drop()



import filecmp
import os.path
import tempfile

from pathlib import Path

import pytest

from snowflake.core.stage import Stage, StageDirectoryTable, StageEncryption
from tests.utils import random_string


CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))


@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
@pytest.mark.usefixtures("skip_for_snowflake_account")
def test_files(stages):
    comment = "my comment"
    new_stage = Stage(
        name=random_string(5, "test_stage_"),
        encryption=StageEncryption(type="SNOWFLAKE_SSE"),
        directory_table=StageDirectoryTable(enable=True),
        comment=comment,
    )
    try:
        s = stages.create(new_stage)
        assert s.fetch().comment == comment
        # upload file
        s.put(CURRENT_DIR + "/../../resources/schema.yaml", "/", auto_compress=False)
        s.put(Path(CURRENT_DIR + "/../../resources/testCSVheader.csv"), "/")

        # list file
        files = list(s.list_files())
        assert len(files) == 2, files
        assert {"schema.yaml", "testCSVheader.csv.gz"} == {f.name.split("/")[-1] for f in files}
        assert [2, 2] == [len(f.name.split("/")) for f in files]

        # download file
        temp_dir = tempfile.mkdtemp()
        s.get("/schema.yaml", temp_dir)
        s.get("/testCSVheader.csv.gz", Path(temp_dir))

        # compare files
        assert filecmp.cmp(CURRENT_DIR + "/../../resources/schema.yaml", os.path.join(temp_dir, "schema.yaml"))
        assert Path(os.path.join(temp_dir, "testCSVheader.csv.gz")).exists()
    finally:
        s.drop()


@pytest.mark.skip_notebook
@pytest.mark.skip_storedproc
@pytest.mark.usefixtures("skip_for_snowflake_account")
def test_files_legacy(stages):
    comment = "my comment"
    new_stage = Stage(
        name=random_string(5, "test_stage_"),
        encryption=StageEncryption(type="SNOWFLAKE_SSE"),
        directory_table=StageDirectoryTable(enable=True),
        comment=comment,
    )
    try:
        s = stages.create(new_stage)
        assert s.fetch().comment == comment
        # upload file
        s.upload_file(CURRENT_DIR + "/../../resources/schema.yaml", "/", auto_compress=False)

        # list file
        files = list(s.list_files())
        assert len(files) == 1, files
        assert "schema.yaml" in files[0].name
        assert 2 == len(files[0].name.split("/"))

        # download file
        temp_dir = tempfile.mkdtemp()
        s.download_file("/schema.yaml", temp_dir)

        # compare files
        assert filecmp.cmp(CURRENT_DIR + "/../../resources/schema.yaml", os.path.join(temp_dir, "schema.yaml"))
    finally:
        s.drop()



from contextlib import suppress

import pytest as pytest

from snowflake.core.exceptions import NotFoundError
from snowflake.core.notification_integration import NotificationEmail, NotificationIntegration
from tests.integ.utils import random_string


# Note: all secret looking things in this file are made up

pytestmark = [
    # All tests in this file need this Snowflake version
    pytest.mark.min_sf_ver("8.36.0"),
    pytest.mark.internal_only,
]


@pytest.fixture()
def notification_integration_extended(notification_integrations, set_internal_params):
    names_list = []
    for _ in range(5):
        names_list.append(random_string(10, "test_notification_integration_iter_a_"))
    for _ in range(7):
        names_list.append(random_string(10, "test_notification_integration_iter_b_"))
    for _ in range(3):
        names_list.append(random_string(10, "test_notification_integration_iter_c_"))

    allowed_recipients = ["test1@snowflake.com", "test2@snowflake.com"]
    default_recipients = ["test1@snowflake.com"]
    default_subject = "test default subject"
    try:
        with set_internal_params({"ENABLE_LIMIT_RECIPIENTS_TO_SENDING_ACCOUNT": False}):
            for ni_name in names_list:
                new_integration = NotificationIntegration(
                    name=ni_name,
                    notification_hook=NotificationEmail(
                        allowed_recipients=allowed_recipients,
                        default_recipients=default_recipients,
                        default_subject=default_subject,
                    ),
                    enabled=True,
                )
                notification_integrations.create(new_integration)

        yield notification_integrations
    finally:
        for ni_name in names_list:
            with suppress(NotFoundError):
                notification_integrations[ni_name].drop()


def test_iter_raw(notification_integration_extended):
    assert len(list(notification_integration_extended.iter())) >= 15


def test_iter_like(notification_integration_extended):
    assert len(list(notification_integration_extended.iter(like="test_view_iter_"))) == 0
    assert len(list(notification_integration_extended.iter(like="test_notification_integration_iter_a_%%"))) == 5
    assert len(list(notification_integration_extended.iter(like="test_notification_integration_iter_b_%%"))) == 7
    assert len(list(notification_integration_extended.iter(like="test_notification_integration_iter_c_%%"))) == 3



from typing import TYPE_CHECKING, Generic, TypeVar

from snowflake.connector import SnowflakeConnection
from snowflake.core._common.base_collection import ObjectCollection


if TYPE_CHECKING:
    from snowflake.core import Root

T = TypeVar("T")


class AccountObjectCollectionParent(ObjectCollection[T], Generic[T]):
    def __init__(self, root: "Root", ref_class: type[T]) -> None:
        super().__init__(ref_class)
        self._root = root

    @property
    def _connection(self) -> SnowflakeConnection:
        return self._root._connection

    @property
    def root(self) -> "Root":
        """The Root object this collection belongs to."""
        return self._root

    def __repr__(self) -> str:
        type_name = type(self).__name__
        return f"<{type_name}>"



from collections.abc import Iterator
from concurrent.futures import Future
from functools import cached_property
from typing import TYPE_CHECKING, Literal, Optional, Union, overload

from pydantic import StrictStr

from snowflake.connector import SnowflakeConnection
from snowflake.core import PollingOperation
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._operation import PollingOperations
from snowflake.core.pipe import PipeCollection
from snowflake.core.schema._generated import SuccessResponse, TagAssignment, TagReference

from .._common import Clone, CreateMode, DatabaseObjectCollectionParent, DatabaseObjectReferenceMixin, PointOfTime
from .._internal.telemetry import api_telemetry
from .._internal.utils import deprecated
from .._utils import tag_assignment_to_tag_tuple, tag_resource_to_tag_reference, tag_tuple_to_tag_assignment
from ..alert import AlertCollection
from ..artifact_repository import ArtifactRepositoryCollection
from ..cortex.search_service import CortexSearchServiceCollection
from ..dynamic_table import DynamicTableCollection
from ..event_table import EventTableCollection
from ..exceptions import NotFoundError
from ..function import FunctionCollection
from ..iceberg_table import IcebergTableCollection
from ..image_repository import ImageRepositoryCollection
from ..network_rule import NetworkRuleCollection
from ..notebook import NotebookCollection
from ..password_policy import PasswordPolicyCollection
from ..procedure import ProcedureCollection
from ..secret import SecretCollection
from ..sequence import SequenceCollection
from ..service import ServiceCollection
from ..stage import StageCollection
from ..stream import StreamCollection
from ..streamlit import StreamlitCollection
from ..table import TableCollection
from ..tag import TagCollection, TagResource, TagValue
from ..task import TaskCollection
from ..user_defined_function import UserDefinedFunctionCollection
from ..view import ViewCollection
from ._generated.api.schema_api import SchemaApi
from ._generated.models.point_of_time import PointOfTime as SchemaPointOfTime
from ._generated.models.schema import SchemaModel as Schema
from ._generated.models.schema_clone import SchemaClone


if TYPE_CHECKING:
    from .. import Root
    from ..database import DatabaseResource


class SchemaCollection(DatabaseObjectCollectionParent["SchemaResource"]):
    """Represents the collection operations on the Snowflake schema resource.

    With this collection, you can create, iterate through, and search for schemas that you have access to in the
    current context.

    Examples
    ________
    Creating a schema instance:

    >>> schemas = root.databases["my_db"].schemas
    >>> new_schema = Schema("my_schema")
    >>> schemas.create(new_schema)
    """

    def __init__(self, database: "DatabaseResource", root: "Root") -> None:
        super().__init__(database, SchemaResource)
        self._api = SchemaApi(
            root=root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(
        self,
        schema: Schema,
        *,
        clone: Optional[Union[str, Clone]] = None,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "SchemaResource":
        """Create a schema in Snowflake.

        Parameters
        __________
        schema: SchemaResource
            The ``Schema`` object, together with the ``Schema``'s properties: name; kind, comment, managed_access,
            retention_time, budget, data_retention_time_in_days, default_ddl_colaltion, log_level,
            pipe_execution_paused, max_data_extension_time_in_days, suspend_task_after_num_failures, trace_level,
            user_task_managed_initial_warehouse_size, user_task_timeout_ms, serverless_task_min_statement_size
            and serverless_task_max_statement_size are optional.
        clone: str, or Clone, optional
            Whether to clone an existing schema. An instance of :class:`Clone`, or str of the name, ``None``
            if no cloning is necessary.
        mode: CreateMode, optional
            One of the following enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the schema already exists in Snowflake.  Equivalent to SQL ``create schema <name> ...``.

            ``CreateMode.or_replace``: Replace if the schema already exists in Snowflake. Equivalent to SQL
            ``create or replace schema <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the schema already exists in Snowflake.
            Equivalent to SQL ``create schema <name> if not exists...``

            Default is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a new schema called ``new_schema`` in ``my_db``:

        >>> schemas = root.databases["my_db"].schemas
        >>> new_schema_ref = schemas.create(Schema("new_schema"))

        Creating a new schema called ``new_schema`` in ``my_db`` by cloning an existing schema:

        >>> schemas = root.databases["my_db"].schemas
        >>> new_schema_ref = schemas.create(
        ...     "new_schema",
        ...     clone=Clone(
        ...         source="original_schema", point_of_time=PointOfTimeOffset(reference="at", when="-5")
        ...     ),
        ...     mode=CreateMode.or_replace,
        ... )

        Creating a new schema called ``new_schema`` in ``my_db`` by cloning an existing schema in another database:

        >>> schemas = root.databases["my_db"].schemas
        >>> new_schema_ref = schemas.create(
        ...     "new_schema",
        ...     clone=Clone(
        ...         source="another_database.original_schema",
        ...         point_of_time=PointOfTimeOffset(reference="at", when="-5"),
        ...     ),
        ...     mode=CreateMode.or_replace,
        ... )
        """
        self._create(schema=schema, clone=clone, mode=mode, async_req=False)
        return self[schema.name]

    @api_telemetry
    def create_async(
        self,
        schema: Schema,
        *,
        clone: Optional[Union[str, Clone]] = None,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["SchemaResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._create(schema=schema, clone=clone, mode=mode, async_req=True)
        return PollingOperation(future, lambda _: self[schema.name])

    @api_telemetry
    def iter(
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        limit: Optional[int] = None,
        from_name: Optional[str] = None,
    ) -> Iterator[Schema]:
        """Iterate through ``Schema`` objects from Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (``%`` and ``_``).
        starts_with: str, optional
            String used to filter the command output based on the string of characters that appear
            at the beginning of the object name. Uses case-sensitive pattern matching.
        limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        from_name: str, optional
            Fetch rows only following the first row whose object name matches
            the specified string. This is case-sensitive and does not have to be the full name.

        Examples
        ________
        Showing all schemas that you have access to see:

        >>> schemas = db_ref.schemas.iter()

        Showing information of the exact schema you want to see:

        >>> schemas = db_ref.schemas.iter(like="your-schema-name")

        Showing schemas starting with 'your-schema-name-':

        >>> schemas = db_ref.schemas.iter(like="your-schema-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for schema in schemas:
        >>>     print(schema.name, schema.query)
        """
        schemas = self._api.list_schemas(
            self.database.name,
            StrictStr(like) if like is not None else None,
            StrictStr(starts_with) if starts_with else None,
            limit,
            from_name=from_name,
            async_req=False,
        )

        return map(Schema._from_model, iter(schemas))

    @api_telemetry
    def iter_async(
        self,
        *,
        like: Optional[str] = None,
        starts_with: Optional[str] = None,
        limit: Optional[int] = None,
        from_name: Optional[str] = None,
    ) -> PollingOperation[Iterator[Schema]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_schemas(
            self.database.name,
            StrictStr(like) if like is not None else None,
            StrictStr(starts_with) if starts_with else None,
            limit,
            from_name=from_name,
            async_req=True,
        )
        return PollingOperation(future, lambda rest_models: map(Schema._from_model, iter(rest_models)))

    @property
    def _connection(self) -> SnowflakeConnection:
        return self.database._connection

    @overload
    def _create(
        self, schema: Schema, clone: Optional[Union[str, Clone]], mode: CreateMode, async_req: Literal[True]
    ) -> Future[SuccessResponse]: ...

    @overload
    def _create(
        self, schema: Schema, clone: Optional[Union[str, Clone]], mode: CreateMode, async_req: Literal[False]
    ) -> SuccessResponse: ...

    def _create(
        self, schema: Schema, clone: Optional[Union[str, Clone]], mode: CreateMode, async_req: bool
    ) -> Union[SuccessResponse, Future[SuccessResponse]]:
        real_mode = CreateMode[mode].value
        model = schema._to_model()

        if clone is not None:
            pot: Optional[SchemaPointOfTime] = None
            if isinstance(clone, Clone) and isinstance(clone.point_of_time, PointOfTime):
                pot = SchemaPointOfTime.from_dict(clone.point_of_time.to_dict())
            real_clone = Clone(source=clone) if isinstance(clone, str) else clone
            req = SchemaClone(point_of_time=pot, **model.to_dict())

            source_schema_path = real_clone.source.split(".")
            source_database = self.database.name
            source_schema = source_schema_path[-1]
            if len(source_schema_path) == 2:
                source_database = source_schema_path[0]
            elif len(source_schema_path) != 1:
                raise NotFoundError(reason=f"{real_clone.source} does not exists", root=self.root)

            return self._api.clone_schema(
                database=source_database,
                name=source_schema,
                schema_clone=req,
                create_mode=StrictStr(real_mode),
                async_req=async_req,
                target_database=self.database.name,
            )

        return self._api.create_schema(
            database=self.database.name, var_schema=model, create_mode=StrictStr(real_mode), async_req=async_req
        )


class SchemaResource(DatabaseObjectReferenceMixin[SchemaCollection]):
    """Represents a reference to a Snowflake schema.

    With this schema reference, you can fetch information about a schema, as well as perform
    certain actions on it.
    """

    def __init__(self, name: str, collection: SchemaCollection) -> None:
        self.name: str = name
        self.collection: SchemaCollection = collection

    @property
    def _api(self) -> SchemaApi:
        return self.collection._api

    @api_telemetry
    @deprecated("create_or_alter")
    def create_or_update(self, schema: Schema) -> "SchemaResource":
        """Create, or update-in-place a schema in Snowflake.

        Parameters
        __________
        schema: SchemaResource
            An instance of :class:`Schema`, the definition of schema we should create.

        Examples
        ________
        Create a schema from a reference:

        >>> my_db.schemas["my_new_schema"].create_or_update(Schema("my_new_schema"))
        """
        return self.create_or_alter(schema=schema)

    @api_telemetry
    def create_or_alter(self, schema: Schema) -> "SchemaResource":
        """Create, or alter-in-place a schema in Snowflake.

        Parameters
        __________
        schema: SchemaResource
            An instance of :class:`Schema`, the definition of schema we should create.

        Examples
        ________
        Create a schema from a reference:

        >>> my_db.schemas["my_new_schema"].create_or_alter(Schema("my_new_schema"))
        """
        self._api.create_or_alter_schema(self.database.name, schema.name, schema._to_model(), async_req=False)
        return self

    @api_telemetry
    def create_or_alter_async(self, schema: Schema) -> PollingOperation["SchemaResource"]:
        """An asynchronous version of :func:`create_or_alter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.create_or_alter_schema(self.database.name, schema.name, schema._to_model(), async_req=True)
        return PollingOperation(future, lambda _: self)

    @api_telemetry
    def fetch(self) -> Schema:
        """Fetch the details of a schema.

        Examples
        ________
        Fetching a reference to a schema to print its name and whether it's our current one.

        >>> my_schema = my_db.schemas["my_schema"].fetch()
        >>> print(my_schema.name, my_schema.is_current)
        """
        return Schema._from_model(self.collection._api.fetch_schema(self.database.name, self.name, async_req=False))

    @api_telemetry
    def fetch_async(self) -> PollingOperation[Schema]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.fetch_schema(self.database.name, self.name, async_req=True)
        return PollingOperation(future, lambda rest_model: Schema._from_model(rest_model))

    @api_telemetry
    @deprecated("drop")
    def delete(self, if_exists: Optional[bool] = None) -> None:
        """Delete this schema.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this schema before dropping it.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Deleting a schema using its reference, erroring if it doesn't exist:

        >>> schema_reference.delete()

        Deleting a schema using its reference, if it exists:

        >>> schema_reference.delete(if_exists=True)
        """
        self.drop(if_exists=if_exists)

    @api_telemetry
    def drop(self, if_exists: Optional[bool] = None) -> None:
        """Drop this schema.

        Parameters
        __________
        if_exists: bool, optional
            Check the existence of this schema before dropping it.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Dropping a schema using its reference, erroring if it doesn't exist:

        >>> schema_reference.drop()

        Dropping a schema using its reference, if it exists:

        >>> schema_reference.drop(if_exists=True)
        """
        self.collection._api.delete_schema(self.database.name, name=self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def drop_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.delete_schema(
            self.database.name, name=self.name, if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def undrop(self) -> None:
        """Undrop this previously dropped schema if it hasn't been purged yet.

        Examples
        ________
        Undropping a schema using its reference:

        >>> schema_reference.drop()
        >>> schema_reference.undrop()
        """
        self.collection._api.undrop_schema(self.database.name, name=self.name, async_req=False)

    @api_telemetry
    def undrop_async(self) -> PollingOperation[None]:
        """An asynchronous version of :func:`undrop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.undrop_schema(self.database.name, name=self.name, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on a schema.

        Parameters
        __________
        tags: dict[TagResource, TagValue]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.set_tags(
            database=self.database.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            database=self.database.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from a schema.

        Parameters
        __________
        tag_resources: set[TagResource]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.unset_tags(
            database=self.database.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            database=self.database.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Get the tag assignments for a schema.

        Returns all tags assigned to a schema. This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Parameter that specifies whether tag assignments inherited by the object from its ancestors in securable
            object hierarchy should be returned as well: - `true`: All tags assigned to this object should be returned,
            inheritance included. - `false`: Only tags explicitly assigned to this object should be returned.
        """
        tag_assignments = self.collection._api.get_tags(
            database=self.database.name,
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(
            database=self.database.name,
            name=self.name,
            with_lineage=with_lineage,
            async_req=True,
        )
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )

    @cached_property
    def alerts(self) -> AlertCollection:
        """The AlertCollection of all alerts contained in this schema.

        Examples
        ________
        Getting all alerts in ``my_schema``:

        >>> my_db.schemas["my_schema"].alerts
        """
        return AlertCollection(self)

    @cached_property
    def event_tables(self) -> EventTableCollection:
        """The EventCollection of all events contained in this schema.

        Examples
        ________
        Getting all events in ``my_schema``:

        >>> my_db.schemas["my_schema"].events
        """
        return EventTableCollection(self)

    @cached_property
    def tasks(self) -> TaskCollection:
        """The TaskCollection of all tasks contained in this schema.

        Examples
        ________
        Getting all tasks in ``my_schema``:

        >>> my_db.schemas["my_schema"].tasks
        """
        return TaskCollection(self)

    @cached_property
    def pipes(self) -> PipeCollection:
        """The PipesCollection of all pipes in this schema.

        Examples
        ________
        Getting all pipes in ``my_schema``:

        >>> my_db.schemas["my_schema"].pipes
        """
        return PipeCollection(self)

    @cached_property
    def services(self) -> ServiceCollection:
        """The ServiceCollection of all services contained in this schema.

        Examples
        ________
        Getting all services in ``my_schema``:

        >>> my_db.schemas["my_schema"].services
        """
        return ServiceCollection(self)

    @cached_property
    def notebooks(self) -> NotebookCollection:
        """The NotebookCollection of all notebooks contained in this schema.

        Examples
        ________
        Getting all notebooks in ``my_schema``:

        >>> my_db.schemas["my_schema"].notebooks
        """
        return NotebookCollection(self)

    @cached_property
    def streamlits(self) -> StreamlitCollection:
        """The StreamlitCollection of all Streamlits contained in this schema.

        Examples
        ________
        Getting all streamlits in ``my_schema``:

        >>> my_db.schemas["my_schema"].streamlits
        """
        return StreamlitCollection(self)

    @cached_property
    def artifact_repositories(self) -> ArtifactRepositoryCollection:
        """The ArtifactRepositoryCollection of all artifact repositories in this schema.

        Examples
        ________
        Getting all artifact repositories in ``my_schema``:

        >>> my_db.schemas["my_schema"].artifact_repositories
        """
        return ArtifactRepositoryCollection(self)

    @cached_property
    def image_repositories(self) -> ImageRepositoryCollection:
        """The ImageRepositoryCollection of all image repositories in this schema.

        Examples
        ________
        Getting all image repositories in ``my_schema``:

        >>> my_db.schemas["my_schema"].image_repositories
        """
        return ImageRepositoryCollection(self)

    @cached_property
    def tables(self) -> TableCollection:
        """The TableCollection of all tables contained in this schema.

        Examples
        ________
        Getting all tables in ``my_schema``:

        >>> my_db.schemas["my_schema"].tables
        """
        return TableCollection(self)

    @cached_property
    def dynamic_tables(self) -> DynamicTableCollection:
        """The DynamicTableCollection of all dynamic tables contained in this schema.

        Examples
        ________
        Getting all dynamic tables in ``my_schema``:

        >>> my_db.schemas["my_schema"].dynamic_tables
        """
        return DynamicTableCollection(self)

    @cached_property
    def iceberg_tables(self) -> IcebergTableCollection:
        """The IcebergTableCollection of all dynamic tables contained in this schema.

        Examples
        ________
        Getting all Iceberg tables in ``my_schema``:

        >>> my_db.schemas["my_schema"].iceberg_tables
        """
        return IcebergTableCollection(self)

    @cached_property
    def stages(self) -> StageCollection:
        """The StageCollection of all stages contained in this schema.

        Examples
        ________
        Getting all stages in ``my_schema``:

        >>> my_db.schemas["my_schema"].stages
        """
        return StageCollection(self)

    @cached_property
    def streams(self) -> StreamCollection:
        """The StreamCollection of all streams contained in this schema.

        Examples
        ________
        Getting all streams in ``my_schema``:

        >>> my_db.schemas["my_schema"].streams
        """
        return StreamCollection(self)

    @cached_property
    def cortex_search_services(self) -> CortexSearchServiceCollection:
        """The CortexSearchServiceCollection of all cortex services contained in this schema.

        Examples
        ________
        Getting all cortex search services in ``my_schema``:

        >>> my_db.schemas["my_schema"].cortex_search_service
        """
        return CortexSearchServiceCollection(self)

    @cached_property
    def functions(self) -> FunctionCollection:
        """The FunctionCollection of all functions contained in this schema.

        Examples
        ________
        Getting all functions in ``my_schema``:

        >>> my_db.schemas["my_schema"].functions
        """
        return FunctionCollection(self)

    @cached_property
    def procedures(self) -> ProcedureCollection:
        """Returns the ``ProcedureCollection`` that represents the visible procedures.

        Examples
        ________
        Getting a specific procedure resource:

        >>> root = Root(session)
        >>> procedure = root.databases["my_db"].schemas["my_schema"].procedures["my_procedure"]
        """
        return ProcedureCollection(self)

    @cached_property
    def views(self) -> ViewCollection:
        """The ViewCollection of all views contained in this schema.

        Examples
        ________
        Getting all views in ``my_schema``:

        >>> my_db.schemas["my_schema"].views
        """
        return ViewCollection(self)

    @cached_property
    def user_defined_functions(self) -> UserDefinedFunctionCollection:
        """The UserDefinedFunctionCollection of all user defined functions contained in this schema.

        Examples
        ________
        Get all user defined functions in ``my_schema``:

        >>> my_db.schemas["my_schema"].user_defined_functions
        """
        return UserDefinedFunctionCollection(self)

    @cached_property
    def network_rules(self) -> NetworkRuleCollection:
        """The NetworkRuleCollection of all network rules contained in this schema.

        Examples
        ________
        Get all network rules in ``my_schema``:

        >>> my_db.schemas["my_schema"].network_rules
        """
        return NetworkRuleCollection(self)

    @cached_property
    def password_policies(self) -> PasswordPolicyCollection:
        """The PasswordPolicyCollection of all password policies contained in this schema.

        Examples
        ________
        Get all password policies in ``my_schema``:

        >>> my_db.schemas["my_schema"].password_policies
        """
        return PasswordPolicyCollection(self)

    @cached_property
    def secrets(self) -> SecretCollection:
        """The SecretCollection of all secrets contained in this schema.

        Examples
        ________
        Get all secrets in ``my_schema``:

        >>> my_db.schemas["my_schema"].secrets
        """
        return SecretCollection(self)

    @cached_property
    def sequences(self) -> SequenceCollection:
        """The SequenceCollection of all sequences contained in this schema.

        Examples
        ________
        Get all sequences in ``my_schema``:

        >>> my_db.schemas["my_schema"].sequences
        """
        return SequenceCollection(self)

    @cached_property
    def tags(self) -> TagCollection:
        """The TagCollection of all tags contained in this schema.

        Examples
        ________
        Get all tags in ``my_schema``:

        >>> my_db.schemas["my_schema"].tags
        """
        return TagCollection(self)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.cortex.embed_service._generated.api.cortex_embed_api import CortexEmbedApi

__all__ = [
    "CortexEmbedApi",
]



from collections.abc import Iterator
from typing import TYPE_CHECKING, Optional

from pydantic import StrictStr

from snowflake.core import PollingOperation
from snowflake.core._common import AccountObjectCollectionParent, ObjectReferenceMixin
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._operation import PollingOperations
from snowflake.core.managed_account._generated.api import ManagedAccountApi
from snowflake.core.managed_account._generated.models.managed_account import ManagedAccountModel as ManagedAccount


if TYPE_CHECKING:
    from snowflake.core import Root


class ManagedAccountCollection(AccountObjectCollectionParent["ManagedAccountResource"]):
    """Represents the collection operations of the Snowflake ManagedAccount resource.

    With this collection, you can create, iterate through, and search for managed accounts that you have access to
    in the current context.

    Examples
    ________
    Creating a managed account instance:

    >>> managed_account_collection = root.managed_accounts
    >>> managed_account = ManagedAccount(
    ...     name="managed_account_name",
    ...     admin_name = "admin"
    ...     admin_password = 'TestPassword1'
    ...     account_type = "READER"
    ...  )
    >>> managed_account_collection.create(managed_account)
    """

    def __init__(self, root: "Root") -> None:
        super().__init__(root, ref_class=ManagedAccountResource)
        self._api = ManagedAccountApi(
            root=root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(self, managed_account: ManagedAccount) -> "ManagedAccountResource":
        """Create a managed account in Snowflake.

        Parameters
        __________
        managed_account: ManagedAccount

        Examples
        ________
        Creating a managed account instance and getting reference to it:

        >>> managed_account_parameters = ManagedAccount(
        ...     name="managed_account_name",
        ...     admin_name = "admin"
        ...     admin_password = 'TestPassword1'
        ...     account_type = "READER"
        ...  )
        >>> # Use the managed account collection created before to create a reference to a managed account resource
        >>> # in Snowflake.
        >>> managed_account_reference = managed_account_collection.create(managed_account_parameters)
        """
        self._api.create_managed_account(managed_account=managed_account._to_model(), async_req=False)
        return self[managed_account.name]

    @api_telemetry
    def create_async(self, managed_account: ManagedAccount) -> PollingOperation["ManagedAccountResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.create_managed_account(managed_account=managed_account._to_model(), async_req=True)
        return PollingOperation(future, lambda _: self[managed_account.name])

    @api_telemetry
    def iter(self, *, like: Optional[str] = None) -> Iterator[ManagedAccount]:
        """Iterate through ``ManagedAccount`` objects in Snowflake, filtering on any optional `like` pattern.

        Parameters
        __________
        like : str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).

        Examples
        ________
        Showing all managed accounts that you have access to see:

        >>> managed_accounts = managed_account_collection.iter()

        Showing information of the exact managed account you want to see:

        >>> managed_accounts = managed_account_collection.iter(like="your-managed-account-name")

        Showing managed accounts starting with 'your-managed-account-name-':

        >>> managed_accounts = managed_account_collection.iter(like="your-managed-account-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for managed_account in managed_accounts:
        >>>     print(managed_account.name, managed_account.comment)
        """
        managed_accounts = self._api.list_managed_accounts(
            StrictStr(like) if like is not None else None, async_req=False
        )

        return map(ManagedAccount._from_model, iter(managed_accounts))

    @api_telemetry
    def iter_async(self, *, like: Optional[str] = None) -> PollingOperation[Iterator[ManagedAccount]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_managed_accounts(StrictStr(like) if like is not None else None, async_req=True)
        return PollingOperation(future, lambda rest_models: map(ManagedAccount._from_model, iter(rest_models)))


class ManagedAccountResource(ObjectReferenceMixin[ManagedAccountCollection]):
    """Represents a reference to a Snowflake managed account.

    With this managed account reference, you can delete a managed account.
    """

    def __init__(self, name: str, collection: ManagedAccountCollection) -> None:
        self.name = name
        self.collection = collection

    @property
    def _api(self) -> ManagedAccountApi:
        """Get the ManagedAccount API object."""
        return self.collection._api

    @api_telemetry
    def drop(self) -> None:
        """Drop this managed account.

        Examples
        ________
        Deleting a warehouse using its reference:

        >>> managed_account_reference.drop()
        """
        self._api.delete_managed_account(self.name, async_req=False)

    @api_telemetry
    def drop_async(self) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.delete_managed_account(self.name, async_req=True)
        return PollingOperations.empty(future)



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.procedure._generated.api.procedure_api import ProcedureApi

__all__ = [
    "ProcedureApi",
]



"""
Snowflake Task API.

The Snowflake Task API is a REST API that you can use to access, update, and perform certain actions on task resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.task._generated.models.minutes_schedule import MinutesSchedule, MinutesScheduleModel
from snowflake.core.task._generated.models.task_schedule import TaskSchedule, TaskScheduleModel


class Task(BaseModel):
    """A model object representing the Task resource.

    Constructs an object of type Task with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    definition : str
        The SQL definition for the task. Any one of single SQL statement, call to stored procedure, or procedural logic using Snowflake scripting.
    warehouse : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    schedule : TaskSchedule, optional

    comment : str, optional
        Specifies a comment for the task.
    finalize : str, optional
        Specifies the name of the root task that the finalizer task is associated with.
    task_auto_retry_attempts : int, optional
        Root task settable only. Specifies the number of automatic task graph retry attempts. Valid range is 0 to 30. When not specified, no retry would happen.
    config : object, optional
        Task Config
    session_parameters : object, optional
        Session Parameters for the task at runtime.
    predecessors : list[str], optional
        Specifies one or more predecessor tasks for the current task
    task_relations : str, optional
        Displays the relationship between the root task and its corresponding finalizer tasks — **Read-only:** *any user-provided value will be ignored.*
    user_task_managed_initial_warehouse_size : str, optional
        Specifies the size of the compute resources to provision for the first run of the task. This parameter only applies to serverless tasks.
    target_completion_interval : MinutesSchedule, optional

    serverless_task_min_statement_size : str, optional
        Specifies the minimum allowed warehouse size for the serverless task. Minimum XSMALL, Maximum XXLARGE. This parameter only applies to serverless tasks.
    serverless_task_max_statement_size : str, optional
        Specifies the maximum allowed warehouse size for the serverless task. Minimum XSMALL, Maximum XXLARGE. This parameter only applies to serverless tasks.
    user_task_timeout_ms : int, optional
        Specifies the time limit on a single run of the task before it times out (in milliseconds).
    suspend_task_after_num_failures : int, optional
        Specifies the number of consecutive failed task runs after which the current task is suspended automatically.
    condition : str, optional
        Specifies a Boolean SQL expression condition; multiple conditions joined with AND/OR are supported
    allow_overlapping_execution : bool, optional
        Specifies whether to allow multiple instances of the DAG to run concurrently.
    error_integration : str, optional
        Specifies the name of the notification integration used to communicate with Amazon SNS, MS Azure Event Grid, or Google Pub/Sub.
    created_on : datetime, optional
        The time the task was created on — **Read-only:** *any user-provided value will be ignored.*
    id : str, optional
        An ID for the current task — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        The role that owns the task — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The role type of the task owner — **Read-only:** *any user-provided value will be ignored.*
    state : str, optional
        The state of the task. Must be one of started or suspended — **Read-only:** *any user-provided value will be ignored.*
    last_committed_on : datetime, optional
        The time the task was last committed on — **Read-only:** *any user-provided value will be ignored.*
    last_suspended_on : datetime, optional
        The time the task was last suspended on — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        The name of the parent database for the task — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        The name of the parent schema for the task — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    warehouse: Optional[Annotated[str, Field(strict=True)]] = None

    schedule: Optional[TaskSchedule] = None

    comment: Optional[StrictStr] = None

    finalize: Optional[StrictStr] = None

    task_auto_retry_attempts: Optional[Annotated[int, Field(le=30, strict=True, ge=0)]] = None

    config: Optional[Dict[str, Any]] = None

    session_parameters: Optional[Dict[str, Any]] = None

    definition: StrictStr

    predecessors: Optional[List[StrictStr]] = None

    task_relations: Optional[StrictStr] = None

    user_task_managed_initial_warehouse_size: Optional[StrictStr] = None

    target_completion_interval: Optional[MinutesSchedule] = None

    serverless_task_min_statement_size: Optional[StrictStr] = None

    serverless_task_max_statement_size: Optional[StrictStr] = None

    user_task_timeout_ms: Optional[StrictInt] = None

    suspend_task_after_num_failures: Optional[StrictInt] = None

    condition: Optional[StrictStr] = None

    allow_overlapping_execution: Optional[StrictBool] = None

    error_integration: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    id: Optional[StrictStr] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    state: Optional[StrictStr] = None

    last_committed_on: Optional[datetime] = None

    last_suspended_on: Optional[datetime] = None

    database_name: Optional[StrictStr] = None

    schema_name: Optional[StrictStr] = None

    __properties = [
        "name",
        "warehouse",
        "schedule",
        "comment",
        "finalize",
        "task_auto_retry_attempts",
        "config",
        "session_parameters",
        "definition",
        "predecessors",
        "task_relations",
        "user_task_managed_initial_warehouse_size",
        "target_completion_interval",
        "serverless_task_min_statement_size",
        "serverless_task_max_statement_size",
        "user_task_timeout_ms",
        "suspend_task_after_num_failures",
        "condition",
        "allow_overlapping_execution",
        "error_integration",
        "created_on",
        "id",
        "owner",
        "owner_role_type",
        "state",
        "last_committed_on",
        "last_suspended_on",
        "database_name",
        "schema_name",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("warehouse")
    def warehouse_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("state")
    def state_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("started", "suspended"):
            raise ValueError("must validate the enum values ('started','suspended')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Task:
        """Create an instance of Task from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "task_relations",
                    "created_on",
                    "id",
                    "owner",
                    "owner_role_type",
                    "state",
                    "last_committed_on",
                    "last_suspended_on",
                    "database_name",
                    "schema_name",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of schedule
        if self.schedule:
            _dict["schedule"] = self.schedule.to_dict()

        # override the default output from pydantic by calling `to_dict()` of target_completion_interval
        if self.target_completion_interval:
            _dict["target_completion_interval"] = self.target_completion_interval.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Task:
        """Create an instance of Task from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Task.model_validate(obj)

        _obj = Task.model_validate(
            {
                "name": obj.get("name"),
                "warehouse": obj.get("warehouse"),
                "schedule": TaskSchedule.from_dict(obj.get("schedule")) if obj.get("schedule") is not None else None,
                "comment": obj.get("comment"),
                "finalize": obj.get("finalize"),
                "task_auto_retry_attempts": obj.get("task_auto_retry_attempts"),
                "config": obj.get("config"),
                "session_parameters": obj.get("session_parameters"),
                "definition": obj.get("definition"),
                "predecessors": obj.get("predecessors"),
                "task_relations": obj.get("task_relations"),
                "user_task_managed_initial_warehouse_size": obj.get("user_task_managed_initial_warehouse_size"),
                "target_completion_interval": MinutesSchedule.from_dict(obj.get("target_completion_interval"))
                if obj.get("target_completion_interval") is not None
                else None,
                "serverless_task_min_statement_size": obj.get("serverless_task_min_statement_size"),
                "serverless_task_max_statement_size": obj.get("serverless_task_max_statement_size"),
                "user_task_timeout_ms": obj.get("user_task_timeout_ms"),
                "suspend_task_after_num_failures": obj.get("suspend_task_after_num_failures"),
                "condition": obj.get("condition"),
                "allow_overlapping_execution": obj.get("allow_overlapping_execution"),
                "error_integration": obj.get("error_integration"),
                "created_on": obj.get("created_on"),
                "id": obj.get("id"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "state": obj.get("state"),
                "last_committed_on": obj.get("last_committed_on"),
                "last_suspended_on": obj.get("last_suspended_on"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
            }
        )

        return _obj


class TaskModel:
    def __init__(
        self,
        name: str,
        definition: str,
        # optional properties
        warehouse: Optional[str] = None,
        schedule: Optional[TaskSchedule] = None,
        comment: Optional[str] = None,
        finalize: Optional[str] = None,
        task_auto_retry_attempts: Optional[int] = None,
        config: Optional[object] = None,
        session_parameters: Optional[object] = None,
        predecessors: Optional[list[str]] = None,
        task_relations: Optional[str] = None,
        user_task_managed_initial_warehouse_size: Optional[str] = None,
        target_completion_interval: Optional[MinutesSchedule] = None,
        serverless_task_min_statement_size: Optional[str] = None,
        serverless_task_max_statement_size: Optional[str] = None,
        user_task_timeout_ms: Optional[int] = None,
        suspend_task_after_num_failures: Optional[int] = None,
        condition: Optional[str] = None,
        allow_overlapping_execution: Optional[bool] = None,
        error_integration: Optional[str] = None,
        created_on: Optional[datetime] = None,
        id: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        state: Optional[str] = None,
        last_committed_on: Optional[datetime] = None,
        last_suspended_on: Optional[datetime] = None,
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
    ):
        """A model object representing the Task resource.

        Constructs an object of type Task with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        definition : str
            The SQL definition for the task. Any one of single SQL statement, call to stored procedure, or procedural logic using Snowflake scripting.
        warehouse : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        schedule : TaskSchedule, optional

        comment : str, optional
            Specifies a comment for the task.
        finalize : str, optional
            Specifies the name of the root task that the finalizer task is associated with.
        task_auto_retry_attempts : int, optional
            Root task settable only. Specifies the number of automatic task graph retry attempts. Valid range is 0 to 30. When not specified, no retry would happen.
        config : object, optional
            Task Config
        session_parameters : object, optional
            Session Parameters for the task at runtime.
        predecessors : list[str], optional
            Specifies one or more predecessor tasks for the current task
        task_relations : str, optional
            Displays the relationship between the root task and its corresponding finalizer tasks.
        user_task_managed_initial_warehouse_size : str, optional
            Specifies the size of the compute resources to provision for the first run of the task. This parameter only applies to serverless tasks.
        target_completion_interval : MinutesSchedule, optional

        serverless_task_min_statement_size : str, optional
            Specifies the minimum allowed warehouse size for the serverless task. Minimum XSMALL, Maximum XXLARGE. This parameter only applies to serverless tasks.
        serverless_task_max_statement_size : str, optional
            Specifies the maximum allowed warehouse size for the serverless task. Minimum XSMALL, Maximum XXLARGE. This parameter only applies to serverless tasks.
        user_task_timeout_ms : int, optional
            Specifies the time limit on a single run of the task before it times out (in milliseconds).
        suspend_task_after_num_failures : int, optional
            Specifies the number of consecutive failed task runs after which the current task is suspended automatically.
        condition : str, optional
            Specifies a Boolean SQL expression condition; multiple conditions joined with AND/OR are supported
        allow_overlapping_execution : bool, optional
            Specifies whether to allow multiple instances of the DAG to run concurrently.
        error_integration : str, optional
            Specifies the name of the notification integration used to communicate with Amazon SNS, MS Azure Event Grid, or Google Pub/Sub.
        created_on : datetime, optional
            The time the task was created on.
        id : str, optional
            An ID for the current task.
        owner : str, optional
            The role that owns the task.
        owner_role_type : str, optional
            The role type of the task owner.
        state : str, optional
            The state of the task. Must be one of started or suspended.
        last_committed_on : datetime, optional
            The time the task was last committed on.
        last_suspended_on : datetime, optional
            The time the task was last suspended on.
        database_name : str, optional
            The name of the parent database for the task.
        schema_name : str, optional
            The name of the parent schema for the task.
        """
        self.name = name
        self.warehouse = warehouse
        self.schedule = schedule
        self.comment = comment
        self.finalize = finalize
        self.task_auto_retry_attempts = task_auto_retry_attempts
        self.config = config
        self.session_parameters = session_parameters
        self.definition = definition
        self.predecessors = predecessors
        self.task_relations = task_relations
        self.user_task_managed_initial_warehouse_size = user_task_managed_initial_warehouse_size
        self.target_completion_interval = target_completion_interval
        self.serverless_task_min_statement_size = serverless_task_min_statement_size
        self.serverless_task_max_statement_size = serverless_task_max_statement_size
        self.user_task_timeout_ms = user_task_timeout_ms
        self.suspend_task_after_num_failures = suspend_task_after_num_failures
        self.condition = condition
        self.allow_overlapping_execution = allow_overlapping_execution
        self.error_integration = error_integration
        self.created_on = created_on
        self.id = id
        self.owner = owner
        self.owner_role_type = owner_role_type
        self.state = state
        self.last_committed_on = last_committed_on
        self.last_suspended_on = last_suspended_on
        self.database_name = database_name
        self.schema_name = schema_name

    __properties = [
        "name",
        "warehouse",
        "schedule",
        "comment",
        "finalize",
        "task_auto_retry_attempts",
        "config",
        "session_parameters",
        "definition",
        "predecessors",
        "task_relations",
        "user_task_managed_initial_warehouse_size",
        "target_completion_interval",
        "serverless_task_min_statement_size",
        "serverless_task_max_statement_size",
        "user_task_timeout_ms",
        "suspend_task_after_num_failures",
        "condition",
        "allow_overlapping_execution",
        "error_integration",
        "created_on",
        "id",
        "owner",
        "owner_role_type",
        "state",
        "last_committed_on",
        "last_suspended_on",
        "database_name",
        "schema_name",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Task(
            name=self.name,
            warehouse=self.warehouse,
            schedule=self.schedule._to_model() if self.schedule is not None else None,
            comment=self.comment,
            finalize=self.finalize,
            task_auto_retry_attempts=self.task_auto_retry_attempts,
            config=self.config,
            session_parameters=self.session_parameters,
            definition=self.definition,
            predecessors=self.predecessors,
            task_relations=self.task_relations,
            user_task_managed_initial_warehouse_size=self.user_task_managed_initial_warehouse_size,
            target_completion_interval=self.target_completion_interval._to_model()
            if self.target_completion_interval is not None
            else None,
            serverless_task_min_statement_size=self.serverless_task_min_statement_size,
            serverless_task_max_statement_size=self.serverless_task_max_statement_size,
            user_task_timeout_ms=self.user_task_timeout_ms,
            suspend_task_after_num_failures=self.suspend_task_after_num_failures,
            condition=self.condition,
            allow_overlapping_execution=self.allow_overlapping_execution,
            error_integration=self.error_integration,
            created_on=self.created_on,
            id=self.id,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            state=self.state,
            last_committed_on=self.last_committed_on,
            last_suspended_on=self.last_suspended_on,
            database_name=self.database_name,
            schema_name=self.schema_name,
        )

    @classmethod
    def _from_model(cls, model) -> TaskModel:
        return TaskModel(
            name=model.name,
            warehouse=model.warehouse,
            schedule=TaskScheduleModel._from_model(model.schedule) if model.schedule else None,
            comment=model.comment,
            finalize=model.finalize,
            task_auto_retry_attempts=model.task_auto_retry_attempts,
            config=model.config,
            session_parameters=model.session_parameters,
            definition=model.definition,
            predecessors=model.predecessors,
            task_relations=model.task_relations,
            user_task_managed_initial_warehouse_size=model.user_task_managed_initial_warehouse_size,
            target_completion_interval=MinutesScheduleModel._from_model(model.target_completion_interval)
            if model.target_completion_interval
            else None,
            serverless_task_min_statement_size=model.serverless_task_min_statement_size,
            serverless_task_max_statement_size=model.serverless_task_max_statement_size,
            user_task_timeout_ms=model.user_task_timeout_ms,
            suspend_task_after_num_failures=model.suspend_task_after_num_failures,
            condition=model.condition,
            allow_overlapping_execution=model.allow_overlapping_execution,
            error_integration=model.error_integration,
            created_on=model.created_on,
            id=model.id,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            state=model.state,
            last_committed_on=model.last_committed_on,
            last_suspended_on=model.last_suspended_on,
            database_name=model.database_name,
            schema_name=model.schema_name,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Task.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Task object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TaskModel:
        """Create an instance of Task from a dict.

        This method constructs a Task object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Task
            A Task object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Task.from_dict(obj))


Task._model_class = TaskModel



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.catalog_integration import (
    CatalogIntegration,
    CatalogIntegrationCollection,
    CatalogIntegrationResource,
    ObjectStore,
)
from snowflake.core.catalog_integration._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
CATALOG_INTEGRATION = CatalogIntegration(
    name="my_catalog_integration", catalog=ObjectStore(), table_format="", enabled=False
)


@pytest.fixture()
def catalog_integrations(fake_root):
    return CatalogIntegrationCollection(fake_root)


@pytest.fixture()
def catalog_integration(catalog_integrations):
    return catalog_integrations["my_catalog_integration"]


def test_create_async(fake_root, catalog_integrations):
    args = (fake_root, "POST", BASE_URL + "/catalog-integrations")
    kwargs = extra_params(
        query_params=[],
        body={
            "name": "my_catalog_integration",
            "catalog": {"catalog_source": "OBJECT_STORE"},
            "table_format": "",
            "enabled": False,
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        catalog_integration_res = catalog_integrations.create(CATALOG_INTEGRATION)
        assert isinstance(catalog_integration_res, CatalogIntegrationResource)
        assert catalog_integration_res.name == "my_catalog_integration"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = catalog_integrations.create_async(CATALOG_INTEGRATION)
        assert isinstance(op, PollingOperation)
        catalog_integration_res = op.result()
        assert catalog_integration_res.name == "my_catalog_integration"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_async(fake_root, catalog_integrations):
    args = (fake_root, "GET", BASE_URL + "/catalog-integrations")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        it = catalog_integrations.iter()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = catalog_integrations.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_async(fake_root, catalog_integration):
    from snowflake.core.catalog_integration._generated.models import CatalogIntegration as CatalogIntegrationModel
    from snowflake.core.catalog_integration._generated.models import ObjectStore as ObjectStoreModel

    model = CatalogIntegrationModel(
        name="my_catalog_integration", catalog=ObjectStoreModel(), table_format="", enabled=False
    )
    args = (fake_root, "GET", BASE_URL + "/catalog-integrations/my_catalog_integration")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        my_catalog_integration = catalog_integration.fetch()
        assert my_catalog_integration.to_dict() == CATALOG_INTEGRATION.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = catalog_integration.fetch_async()
        assert isinstance(op, PollingOperation)
        my_catalog_integration = op.result()
        assert my_catalog_integration.to_dict() == CATALOG_INTEGRATION.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_async(fake_root, catalog_integration):
    args = (fake_root, "DELETE", BASE_URL + "/catalog-integrations/my_catalog_integration")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        catalog_integration.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = catalog_integration.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, catalog_integration, tag):
    args = (fake_root, "POST", BASE_URL + "/catalog-integrations/my_catalog_integration:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        catalog_integration.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = catalog_integration.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, catalog_integration, tag):
    args = (fake_root, "POST", BASE_URL + "/catalog-integrations/my_catalog_integration:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        catalog_integration.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = catalog_integration.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, catalog_integration):
    args = (fake_root, "GET", BASE_URL + "/catalog-integrations/my_catalog_integration:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert catalog_integration.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = catalog_integration.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



import pytest

from snowflake.core.streamlit import (
    AddVersionFromGitStreamlitRequest,
    GitPushWithUsernamePassword,
    Streamlit,
    StreamlitVersionForGit,
)
from tests.integ.utils import random_string


@pytest.mark.min_sf_ver("9.38.0")
def test_push_and_pull_streamlit(streamlits, streamlit_stage_with_file, warehouse, streamlit_main_file, git_repository):
    if not git_repository.get("commit_hash") or not git_repository.get("remote_url"):
        pytest.skip(
            "Git not available or no remote repository. Set TEST_GIT_REPO_URL, GITHUB_TOKEN, or use GitHub CLI."
        )

    name = random_string(8, "test_push_pull_streamlit_")

    st = Streamlit(
        name=name,
        query_warehouse=warehouse.name,
        source_location=f"@{streamlit_stage_with_file.name}",
        main_file=streamlit_main_file,
    )

    ref = streamlits.create(st)
    try:
        version_name = random_string(8, "git_version_")
        version = StreamlitVersionForGit(
            name=version_name,
            comment="Initial Git version for push/pull test",
        )

        git_ref = f"{git_repository['remote_url']}@{git_repository['commit_hash']}"
        add_version_from_git_request = AddVersionFromGitStreamlitRequest(
            version=version,
            git_ref=git_ref,
        )

        ref.add_version_from_git(add_version_from_git_request)

        initial_fetched = ref.fetch()
        assert initial_fetched.last_version_details is not None
        assert initial_fetched.last_version_details.name.upper() == version_name.upper()

        push_options = GitPushWithUsernamePassword(
            git_username="test_user",
            git_password="test_password",
            git_author_name="Test Author",
            git_author_email="test@example.com",
            to_git_branch_uri=f"{git_repository['remote_url']}@main",
            git_push_comment="Test push from integration test",
        )

        # Capture state before push for comparison
        before_push_fetched = ref.fetch()
        before_push_version = before_push_fetched.last_version_details

        ref.push(push_options)

        # Verify push succeeded and streamlit state is intact
        after_push_fetched = ref.fetch()
        assert after_push_fetched.name.upper() == name.upper()
        assert after_push_fetched.last_version_details is not None

        # Verify version details are valid after push
        assert after_push_fetched.last_version_details.name is not None
        assert after_push_fetched.last_version_details.alias is not None

        # Ensure version consistency (name might be updated but should exist)
        if before_push_version:
            assert (
                after_push_fetched.last_version_details.name == before_push_version.name
                or after_push_fetched.last_version_details.name is not None
            )

            # Source location should remain consistent
            if before_push_version.source_location_uri:
                assert (
                    after_push_fetched.last_version_details.source_location_uri
                    == before_push_version.source_location_uri
                    or after_push_fetched.last_version_details.source_location_uri is not None
                )

        # Test pull operation (should fail if already up to date)
        try:
            ref.pull()
            pull_successful = True
        except Exception as pull_error:
            pull_successful = False
            error_msg = str(pull_error).lower()
            is_up_to_date_error = any(
                phrase in error_msg
                for phrase in [
                    "up to date",
                    "already up to date",
                    "nothing to pull",
                    "no changes",
                    "current version",
                ]
            )
            if not is_up_to_date_error:
                raise pull_error

        # Verify pull succeeded and streamlit state is intact
        final_fetched = ref.fetch()
        assert final_fetched.name.upper() == name.upper()

        # Verify version details are still present after pull
        assert final_fetched.last_version_details is not None
        assert final_fetched.last_version_details.name is not None
        assert final_fetched.last_version_details.alias is not None

        # If pull was successful, verify version consistency
        if pull_successful:
            assert (
                final_fetched.last_version_details.name.upper() == version_name.upper()
                or final_fetched.last_version_details.name is not None
            )

    finally:
        ref.drop()



from typing import TYPE_CHECKING

from snowflake.core import PollingOperation
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._internal.utils import deprecated
from snowflake.core.cortex.lite_agent_service._generated.api import CortexLiteAgentApi
from snowflake.core.cortex.lite_agent_service._generated.models import AgentRunRequest
from snowflake.core.rest import SSEClient


if TYPE_CHECKING:
    from snowflake.core import Root


class CortexAgentService:
    """Represents the operations of the Snowflake Cortex Lite Agent Service resource."""

    def __init__(self, root: "Root") -> None:
        self._api = CortexLiteAgentApi(root=root, resource_class=None, sproc_client=StoredProcApiClient(root=root))

    @api_telemetry
    def run(self, agent_run_request: AgentRunRequest) -> SSEClient:
        """Perform agent service.

        Parameters
        __________
        agent_request: AgentRequest
            The agent request object to be sent to the agent service.
            Defined in ./_generated/models/agent_run_request.py
        """
        return SSEClient(self._api.agent_run(agent_run_request, async_req=False, _preload_content=False))

    @api_telemetry
    def run_async(self, agent_run_request: AgentRunRequest) -> PollingOperation[SSEClient]:
        """An asynchronous version of :func:`Run`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.agent_run(agent_run_request, async_req=True, _preload_content=False)
        return PollingOperation(future, lambda x: SSEClient(x))

    @api_telemetry
    @deprecated("run")
    def Run(self, agent_run_request: AgentRunRequest) -> SSEClient:
        return self.run(agent_run_request=agent_run_request)



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#

import pytest

from snowflake.core.exceptions import NotFoundError

from ..utils import random_object_name


def test_drop_basic(tasks, root):
    tasks["dummy______task"].drop(if_exists=True)
    task_name = random_object_name()
    try:
        create_task = (
            f"create or replace task {task_name} "
            "ALLOW_OVERLAPPING_EXECUTION = true SUSPEND_TASK_AFTER_NUM_FAILURES = 10 "
            "schedule = '10 minute' as select current_version()"
        )
        root.connection.execute_string(create_task)
        tasks[task_name].drop()
        assert len(list(tasks.iter(like=task_name))) == 0
    finally:
        root.connection.execute_string(f"drop task if exists {task_name}")

    with pytest.raises(NotFoundError):
        tasks[random_object_name()].drop()



import pytest

import snowflake.core._internal.telemetry


pytest_plugins = ["tests.integ.deflake"]


def pytest_configure(config):
    snowflake.core._internal.telemetry._called_from_test = True


def pytest_collection_modifyitems(items):
    for item in items:
        # Mark every test that uses session to need SnowPark
        if "session" in item.fixturenames:
            item.add_marker(pytest.mark.snowpark)



from contextlib import suppress

import pytest

from snowflake.core.alert import Alert, MinutesSchedule
from snowflake.core.exceptions import NotFoundError
from tests.utils import random_string

from ...utils import ensure_snowflake_version


@pytest.fixture(scope="module")
def alerts_extended(alerts, snowflake_version):
    ensure_snowflake_version(snowflake_version, "8.36.0")

    name_list = []
    for _ in range(5):
        name_list.append(random_string(10, "test_alert_iter_a_"))
    for _ in range(7):
        name_list.append(random_string(10, "test_alert_iter_b_"))
    for _ in range(3):
        name_list.append(random_string(10, "test_alert_iter_c_"))

    for alert_name in name_list:
        alerts.create(
            Alert(
                name=alert_name,
                condition="SELECT 1",
                action="SELECT 2",
                schedule=MinutesSchedule(minutes=1),
                comment="ThIs iS a ComMeNT",
            )
        )

    try:
        yield alerts
    finally:
        for alert_name in name_list:
            with suppress(NotFoundError):
                alerts[alert_name].drop()


def test_iter_raw(alerts_extended):
    assert len(list(alerts_extended.iter())) >= 15


def test_iter_like(alerts_extended):
    assert len(list(alerts_extended.iter(like="test_alert_iter_"))) == 0
    assert len(list(alerts_extended.iter(like="test_alert_iter_a_%%"))) == 5
    assert len(list(alerts_extended.iter(like="test_alert_iter_b_%%"))) == 7
    assert len(list(alerts_extended.iter(like="test_alert_iter_c_%%"))) == 3


def test_iter_show_limit(alerts_extended):
    assert len(list(alerts_extended.iter(like="test_alert_iter_a_%%"))) == 5
    assert len(list(alerts_extended.iter(like="test_alert_iter_a_%%", show_limit=2))) == 2
    assert len(list(alerts_extended.iter(show_limit=2))) == 2


def test_iter_starts_with(alerts_extended):
    assert len(list(alerts_extended.iter(starts_with="test_alert_iter_a_".upper()))) == 5


def test_iter_from_name(alerts_extended):
    assert len(list(alerts_extended.iter(from_name="test_alert_iter_b_"))) >= 10



from io import BytesIO
from textwrap import dedent

import pytest

from pydantic import StrictStr

from snowflake.core.service import Service, ServiceSpecStageFile

from ..utils import random_string


@pytest.fixture(scope="session")
def temp_service_for_function(services, session, imagerepo, shared_compute_pool) -> StrictStr:
    stage_name = random_string(5, "test_stage_ff_")
    s_name = random_string(5, "test_service_ff_")
    session.sql(f"create temp stage {stage_name};").collect()
    spec_file = "spec.yaml"
    spec = f"@{stage_name}/{spec_file}"
    session.file.put_stream(
        BytesIO(
            dedent(
                f"""
                spec:
                    containers:
                        - name: hello-world
                          image: {imagerepo}/hello-world:latest
                    endpoints:
                        - name: ep1
                          port: 8000
                        - name: end-point-2
                          port: 8010
                 """
            ).encode()
        ),
        spec,
    )

    test_s = Service(
        name=s_name,
        compute_pool=shared_compute_pool,
        spec=ServiceSpecStageFile(stage=stage_name, spec_file=spec_file),
        min_instances=1,
        max_instances=1,
        comment="created by temp_service for function",
    )
    s = services.create(test_s)
    try:
        yield s
    finally:
        session.sql(f"DROP SERVICE IF EXISTS {s.name}").collect()



"""Manages Snowflake Tables."""

from ..table._generated.models import Constraint, ForeignKey, PrimaryKey, Table, TableColumn, UniqueKey
from ._table import TableCollection, TableResource


__all__ = [
    "Table",
    "TableResource",
    "TableCollection",
    "TableColumn",
    "ForeignKey",
    "PrimaryKey",
    "UniqueKey",
    "Constraint",
]



import pytest

from snowflake.core._internal.root_configuration import RootConfiguration


def test_root_config():
    test_root_config = RootConfiguration()
    assert test_root_config.get_user_agents() == ""
    assert not test_root_config.has_user_agents()

    with pytest.raises(ValueError):
        test_root_config.append_user_agent("python_api")
    with pytest.raises(ValueError):
        test_root_config.append_user_agent("py%", "1.2")
    with pytest.raises(ValueError):
        test_root_config.append_user_agent("py", "/1.")

    test_root_config.append_user_agent("Snowflake", "1.2.3")
    assert test_root_config.get_user_agents() == "Snowflake/1.2.3"
    assert test_root_config.has_user_agents()

    test_root_config.append_user_agent("custom_ui")
    assert test_root_config.get_user_agents() == "Snowflake/1.2.3 custom_ui"
    assert test_root_config.has_user_agents()

    test_root_config = RootConfiguration()
    test_root_config.append_user_agent("custom_ui")
    assert test_root_config.get_user_agents() == "custom_ui"
    assert test_root_config.has_user_agents()



#
# Copyright (c) 2012-2023 Snowflake Computing Inc. All rights reserved.
#


import pytest

from snowflake.core._common import CreateMode
from snowflake.core.exceptions import APIError, ConflictError
from snowflake.core.image_repository import ImageRepository
from tests.utils import random_string


def test_create(image_repositories):
    ir_names = []
    try:
        ir_name = random_string(5, "test_ir_")
        ir_names += [ir_name]
        test_ir = ImageRepository(name=ir_name)
        image_repo = image_repositories.create(test_ir)
        ir = image_repo.fetch()
        assert ir.name == ir_name.upper()
        image_repo.drop()

        ir_name = random_string(5, "test_ir_")
        ir_names += [ir_name]
        ir_name = f'"{ir_name}"'
        test_ir = ImageRepository(name=ir_name)
        # image repository with case sensitive names is not supported
        with pytest.raises(APIError):
            image_repo = image_repositories.create(test_ir)

        ir_name = random_string(5, "test_ir_")
        ir_names += [ir_name]
        test_ir = ImageRepository(name=ir_name)
        image_repo = image_repositories.create(test_ir)

        # create an already existing image repository with errorifExists
        with pytest.raises(ConflictError):
            image_repo = image_repositories.create(test_ir)

        # create an already existing image repository with ifNotExists
        image_repo = image_repositories.create(test_ir, mode=CreateMode.if_not_exists)
        # create when we do or replace
        image_repo = image_repositories.create(test_ir, mode=CreateMode.or_replace)

        # create from fetch
        fetched_ir = image_repo.fetch()
        image_repositories.create(fetched_ir, mode=CreateMode.or_replace)

    finally:
        for ir_name in ir_names:
            image_repositories[ir_name].drop(if_exists=True)
